package com.lyms.talkonlineweb.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.lyms.talkonlineweb.annotation.TokenRequired;
import com.lyms.talkonlineweb.domain.*;
import com.lyms.talkonlineweb.result.BaseResponse;
import com.lyms.talkonlineweb.service.*;
import com.lyms.talkonlineweb.util.Constant;
import com.lyms.talkonlineweb.util.JwtUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 科室管理
 */
@RestController
@RequestMapping("depart")
public class DepartController {

    @Autowired
    private LymsHdepartService lymsHdepartService;

    @Autowired   //科室所拥有的疾病
    private LymsDepartillService lymsDepartillService;

    @Autowired
    private HdepartInfoService hdepartInfoService;

    @Autowired
    private LymsLogsCrudService lymsLogsCrudService;


    /**
     * 获取科室列表
     * @param depart
     * @param current
     * @param size
     * @return
     */
    @GetMapping("sltDepartLst")
    @TokenRequired
    public BaseResponse sltDepartLst(HdepartInfo depart, int current, int size){
        BaseResponse baseResponse=new BaseResponse();
//        Page<LymsHdepart> page=new Page<>(current,size);
//        Page<LymsHdepart> departPage=lymsHdepartService.page(page, Wrappers.query(depart).orderByDesc("updated_time","createdtime"));
//        addIll(departPage);
        Page<HdepartInfo> page=new Page<>(current,size);
        Page<HdepartInfo> departPage=hdepartInfoService.page(page, Wrappers.query(depart).orderByDesc("createdtime"));
        addIll(departPage);
        baseResponse.setObject(departPage);

        return baseResponse;
    }

    public void addIll(Page<HdepartInfo> departPage){
        departPage.getRecords().forEach(e->{
            Map param=new HashMap();
            param.put("did",e.getDid());
            List<LymsDepartill> iLst=lymsDepartillService.listByMap(param);
            iLst.forEach(i->{
                if(e.getIlls()==null){
                    e.setIlls("");
                    e.setIllsName("");
                }
                e.setIlls(e.getIlls()+i.getIid()+",");
                e.setIllsName(e.getIllsName()+i.getIname()+",");
            });
        });
    }



    /**
     * 保存或更新科室信息
     * @param depart
     * @return
     */
    @PostMapping("saveDepart")
    @TokenRequired
    public BaseResponse saveDepart(@RequestBody LymsHdepart depart, BindingResult result){
        BaseResponse baseResponse=new BaseResponse();
        baseResponse.setErrormsg("");
        //操作记录标识
        Integer hid=0;
        if(depart.getDid()==null){
            depart.setCreatedtime(new Date());
            LymsHdepart depart2=new LymsHdepart();

        }else{
            hid=1;
            depart.setUpdatedTime(new Date());
        }
        if(result.hasErrors()){
            baseResponse.setErrorcode(1);
            result.getAllErrors().forEach(e->{
                baseResponse.setErrormsg(baseResponse.getErrormsg()+e.getDefaultMessage());
            });

            return baseResponse;
        }
        boolean f=lymsHdepartService.saveOrUpdate(depart);

        Map param=new HashMap();
        param.put("did",depart.getDid());
        lymsDepartillService.removeByMap(param);

        String[] iArr={};
        if(depart.getIlls()!=null){
            iArr=depart.getIlls().split(",");
        }
        String[] iArrName={};
        if(depart.getIllsName()!=null){
            iArrName=depart.getIllsName().split(",");
        }

        for (int i = 0; i < iArr.length; i++) {
            LymsDepartill departill=new LymsDepartill();
            departill.setDid(depart.getDid());
            departill.setIid(Integer.parseInt(iArr[i]));
            departill.setIname(iArrName[i]);
            lymsDepartillService.save(departill);
        }
        //保存操作记录
        lymsLogsCrud(depart,hid);

        baseResponse.setErrorcode(f==true?0:1);
        return baseResponse;
    }

    /**
     * 删除科室
     * @param did
     * @return
     */
    @GetMapping("delDepart")
    @TokenRequired
    public BaseResponse delHosp(int did){
        BaseResponse baseResponse=new BaseResponse();
        //操作记保存参数
        LymsHdepart depart=lymsHdepartService.getById(did);

        boolean f=lymsHdepartService.removeById(did);
        //保存操作记录
        lymsLogsCrud(depart,null);

        baseResponse.setErrorcode(f==true?0:1);
        return baseResponse;
    }


    /**
     * 科室人员或者护士登录
     * @param depart
     * @return
     */
    @PostMapping("loginDepart")
    @TokenRequired
    public BaseResponse loginDepart(LymsHdepart depart){
        BaseResponse baseResponse=new BaseResponse();
        List<LymsHdepart> dLst=lymsHdepartService.list(Wrappers.query(depart));

//        if (dLst.size()>0) {
//            depart=dLst.get(0);
//            depart.setDpasswd(null);
//            String jwt = JwtUtils.createJWT("1", depart.getDusername(), Constant.JWT_TTL);
//            Map<String,Object> map=new HashMap<>();
//            map.put("depart",depart);
//            map.put("token",jwt);
//            baseResponse.setObject(map);
//        }

        return baseResponse;
    }


    /**
     * 操作记录
     * @param depart
     * @param hid
     */
    public void lymsLogsCrud(LymsHdepart depart,Integer hid){
        LymsLogsCrud lymsLogsCrud=new LymsLogsCrud();
        if(null!=hid){
            lymsLogsCrud.setCrudtype(hid==1?"编辑科室":"新增科室");
        }else {
            lymsLogsCrud.setCrudtype("删除科室");
        }
        lymsLogsCrud.setCreatedbyname(depart.getCreator());
        lymsLogsCrud.setUpdatedbyname(depart.getUpdatetor());
        lymsLogsCrud.setHname(depart.getHname());
        lymsLogsCrud.setHdname(depart.getDname());
        lymsLogsCrud.setCreatdata(new Date());
        lymsLogsCrud.setType(1);
        lymsLogsCrudService.save(lymsLogsCrud);

    }

    /**
     * PC-查询同一家医院科室名称是否重复
     * @param   hid 所属医院
     * @param   dname 科室名称
     * @param  did 修改时用
     * @return true 可以使用，false已经存在
     */
    @GetMapping("getDepartByParam")
    @TokenRequired
    public BaseResponse getDepartByParam(Integer hid,String dname,@RequestParam(required = false) Integer did){
        BaseResponse baseResponse=new BaseResponse();
        try {
            QueryWrapper<LymsHdepart> queryWrapper=new QueryWrapper<>();
            queryWrapper.eq("hid", hid);
            queryWrapper.eq("dname", dname);
            if(null!=did){//修改的时候判断是否重复
                QueryWrapper<LymsHdepart> queryWrapper2=new QueryWrapper<>();
                queryWrapper2.eq("did", did);
                LymsHdepart lymsHdepart2=lymsHdepartService.getOne(queryWrapper2);
                LymsHdepart lymsHdepart=lymsHdepartService.getOne(queryWrapper);
                baseResponse.setObject(null==lymsHdepart?true:(lymsHdepart2.getDname().equals(dname)?true:false));
            }else {
                LymsHdepart lymsHdepart=lymsHdepartService.getOne(queryWrapper);
                baseResponse.setObject(null==lymsHdepart?true:false);
            }
            baseResponse.setErrormsg("成功");
        } catch (Exception e) {
            baseResponse.setErrormsg("失败");
            e.printStackTrace();
        }
        return baseResponse;
    }

}
