package com.lyms.talkonlineweb.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.lyms.talkonlineweb.annotation.Resubmit;
import com.lyms.talkonlineweb.annotation.TokenRequired;
import com.lyms.talkonlineweb.domain.*;
import com.lyms.talkonlineweb.enums.PayStatus;
import com.lyms.talkonlineweb.result.BaseResponse;
import com.lyms.talkonlineweb.service.*;
import com.lyms.talkonlineweb.util.*;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

/**
 * @ProjectName: talkonline
 * @Package: com.lyms.talkonlineweb.controller
 * @ClassName: GroupOrderController
 * @Author: sy
 * @Description: 支付订单类
 * @Date: 2022-04-22 09:57
 * @Version:
 */
@RestController
@RequestMapping("GroupOrder")
@Log4j2
public class GroupOrderController {

    /**
     * 商品信息
     */
    @Autowired
    private LymsGroupGoodsService lymsGroupGoodsService;

    /**
     * 患者
     */
    @Autowired
    private LymsPatientService lymsPatientService;

    /**
     * 订单信息
     */
    @Autowired
    private LymsGroupOrderService lymsGroupOrderService;

    /**
     * 订单量
     */
    @Autowired
    private LymsOrderDataService lymsOrderDataService;

    /**
     * 微信回调通知接口
     */
    @Value("${notify.url}")
    private String notifyUrl;
    /**
     * 终端ip
     */
    @Value("${weixin.createIP}")
    private String createIP;


    /**
     * 创建支付订单，并把订单提交到微信平台
     * 传入患者id和购买数量
     * https://pay.weixin.qq.com/wiki/doc/api/wxa/wxa_api.php?chapter=9_1  微信统一下单api文档地址
     * @param lymsOrder
     * @param request
     * @param response
     */
    @PostMapping("createPayOrder")
    @TokenRequired
    @Resubmit
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse createPayOrder(@RequestBody LymsGroupOrder lymsOrder,
                                       HttpServletRequest request, HttpServletResponse response) {

        log.info("create pay order request info : {}",lymsOrder.toString());

        BaseResponse baseResponse = new BaseResponse();
        //参数校验
        if (lymsOrder.getPid() == null || CollectionUtils.isEmpty(lymsOrder.getOrderData()))
        {
            baseResponse.setErrorcode(1);
            baseResponse.setErrormsg("创建订单参数错误");
            return baseResponse;
        }
        for (LymsOrderData orderData : lymsOrder.getOrderData()) {
            if (null==orderData.getGid() || null==orderData.getOrderNum()) {
                baseResponse.setErrorcode(1);
                baseResponse.setErrormsg("创建订单参数错误");
                return baseResponse;
            }
        }
        LymsPatient patient = lymsPatientService.getById(lymsOrder.getPid());
        if (patient == null)
        {
            baseResponse.setErrorcode(1);
            baseResponse.setErrormsg("用户不存在");
            return baseResponse;
        }
        if (StringUtil.isEmpty(patient.getOpenid()))
        {
            baseResponse.setErrorcode(1);
            baseResponse.setErrormsg("用户Openid不存在");
            return baseResponse;
        }

        try
        {
            //生成系统订单号
            String orderNo = StringUtil.getUniqueNo();
            String wieXinOrderNo = StringUtil.getUniqueWeixNo();
            lymsOrder.setPaytime(new Date());
            lymsOrder.setOrderno(orderNo);
            lymsOrder.setPayorderid(wieXinOrderNo);
            lymsOrder.setStatus(PayStatus.CREATED.getCode());
            lymsOrder.setOpenid(patient.getOpenid());
            lymsGroupOrderService.save(lymsOrder);
            //处理订单量&更新订单
            Integer total= 0;
            for (LymsOrderData orderData : lymsOrder.getOrderData()) {
                LambdaQueryWrapper<LymsGroupGoods> wrapper = new QueryWrapper().lambda();
                wrapper.eq(LymsGroupGoods::getId, orderData.getGid());
                LymsGroupGoods goods = lymsGroupGoodsService.getOne(wrapper);
                if (goods == null || goods.getPrice() <= 0 )
                {
                    baseResponse.setErrorcode(1);
                    baseResponse.setErrormsg("商品价格配置不存在");
                    return baseResponse;
                }
                if(null ==lymsOrder.getId()){
                    baseResponse.setErrorcode(1);
                    baseResponse.setErrormsg("订单信息id获取异常！");
                    return baseResponse;
                }
                orderData.setOrderId(lymsOrder.getId());
                orderData.setGid(goods.getId());
                orderData.setStartTime(new Date());
                orderData.setEndTime(DateUtil.addDay(new Date(), orderData.getOrderNum()*goods.getTimeLimit()));
                total+=orderData.getOrderNum() * goods.getPrice();
                lymsOrderDataService.save(orderData);
            }
            lymsOrder.setTotalPrices(total);
            //创建微信订单
            Map<String, String> data = createWxOrder(lymsOrder);
            if (data != null) {
                //更新订单总金额
                lymsGroupOrderService.updateById(lymsOrder);
                data.put("orderNo", orderNo);
                baseResponse.setObject(data);
                return baseResponse;
            }

        }catch (Exception e){
            log.error("create order error.",e);
        }
        baseResponse.setErrorcode(1);
        baseResponse.setErrormsg("创建订单失败");
        return baseResponse;
    }

    /**
     * 创建微信订单
     */
    private Map<String,String> createWxOrder(LymsGroupOrder lymsOrder) throws Exception
    {
        try {
            String xml =  buildRequestXml(lymsOrder);
            log.info("create order pid : {},buildRequestMap xml : {}",lymsOrder.getPid(),xml);

            //调用微信统一下单接口
            String result = HttpUtil.postData(Constant.PAY_URL, xml);
            log.info("wx create order result : {}" ,result);

            return parseWxResult(result);
        } catch (Exception e) {
           log.error("createWxOrder error.",e);
           throw e;
        }
    }


    /**
     * 解析微信创建订单的结果
     * @param resultXml
     * @throws Exception
     */
    private Map<String,String> parseWxResult(String resultXml) throws Exception{
        try
        {
            Map<String,String> data = new HashMap(5);
            Map<String,String> retMap = XmlUtil.xmlToMap(resultXml);
            System.out.println("retMap:"+retMap);
            String success = "SUCCESS";
            if (retMap != null && retMap.size() > 0 && success.equals(retMap.get("return_code")) && success.equals(retMap.get("result_code")))
            {
                SortedMap<Object,Object> map=new TreeMap<>();
                //重新拼接支付sign
                String nonceStr=NumberUtil.getRandomString(16);
                String timeStamp=System.currentTimeMillis()/1000+"";
                map.put("appId",Constant.PAT_APP_ID);
                map.put("nonceStr",nonceStr);
                map.put("package","prepay_id="+retMap.get("prepay_id"));
                map.put("signType","MD5");
                map.put("timeStamp",timeStamp);
                //System.out.println("createSign:"+createSign(null, map));
                //返给前端加密后的sign,
                data.put("packageValue",retMap.get("prepay_id"));
                data.put("nonceStr",nonceStr);
                data.put("paySign",createSign(null, map));
                data.put("timeStamp",timeStamp);
                return data;
            }
        }
        catch (Exception e)
        {
            log.error("parse wx result error.",e);
            throw  e;
        }
        return null;
    }


    /**
     * 构建微信订单参数
     * @param lymsOrder
     * @return
     * @throws Exception
     */
    public String buildRequestXml(LymsGroupOrder lymsOrder) throws Exception {
        Map<String,String> paramMap = new TreeMap((key1,key2) -> {
                return key1.toString().compareTo(key2.toString());
             }
        );
        //小程序ID
        paramMap.put("appid", Constant.PAT_APP_ID);
        //商户号
        paramMap.put("mch_id", Constant.MCHID);
        //随机字符串
        paramMap.put("nonce_str",   NumberUtil.getRandomString(16));
        //签名类型
        paramMap.put("sign_type",  "MD5");
        //商户订单号
        paramMap.put("out_trade_no",  lymsOrder.getPayorderid());
        //标价金额
        paramMap.put("total_fee",  String.valueOf(lymsOrder.getTotalPrices()));
        //终端IP
        paramMap.put("spbill_create_ip",  createIP);
        //通知地址
        paramMap.put("notify_url",  notifyUrl);
        //交易类型
        paramMap.put("trade_type",  "JSAPI");
        //用户标识
        paramMap.put("openid", lymsOrder.getOpenid());
        //商品描述
        paramMap.put("body",  "问诊支付");

        String reqSign = PayDigestUtil.getSign(paramMap, Constant.REQ_KEY);
        // 签名
        paramMap.put("sign", reqSign);
        return XmlUtil.mapToXml(paramMap);
    }


    /**
     * 微信支付成功后回调接口
     * https://pay.weixin.qq.com/wiki/doc/api/wxa/wxa_api.php?chapter=9_7&index=8
     * @param request
     * @param response
     * @return
     */
    @PostMapping("payNotify")
    public void payNotify(HttpServletRequest request, HttpServletResponse response) {

        Map<String,String> result =  new HashMap<>(2);
        result.put("return_code","<![CDATA[FAIL]]>");
        result.put("return_msg","参数错误");
        try {
            Map<String, String> paramMap = getPayNotifyParamMap(request);

            //验证支付通知的参数合法性
            if (!verifyPayNotify(paramMap)) {
                //微信支付订单号
                String payOrderId =  paramMap.get("transaction_id");
                //商户订单号
                String mchOrderNo =  paramMap.get("out_trade_no");
                LambdaQueryWrapper<LymsGroupOrder> wrapper = new QueryWrapper().lambda();
                wrapper.eq(LymsGroupOrder::getOrderno, mchOrderNo);
                LymsGroupOrder order=new LymsGroupOrder();
                order.setPayorderid(payOrderId);
                boolean f=lymsGroupOrderService.update(order, wrapper);

            }
            outResult(response, XmlUtil.mapToXml(result));
        }catch (Exception e){
            log.error("pay notify error.",e);
            try
            {
                outResult(response, XmlUtil.mapToXml(result));
            }catch (Exception e1) {

            }
        }
    }


    /**
     * 回调接口响应微信
     * @param response
     * @param content
     */
    private void outResult(HttpServletResponse response, String content) {
        response.setContentType("text/html");
        PrintWriter pw;
        try {
            pw = response.getWriter();
            pw.print(content);
            log.info("response  pay complete.");
        } catch (IOException e) {
            log.error("response pay write exception.", e);
        }
    }


    /**
     * 解析支付通知参数为map
     * @param request
     * @return
     * @throws Exception
     */
    public Map<String, String> getPayNotifyParamMap(HttpServletRequest request) throws Exception{
        try
        {
            String resultXml = IOUtils.toString(request.getInputStream(),"utf-8");
            log.info("pay notify info : {}",resultXml);
            if (StringUtil.isNotEmpty(resultXml))
            {
                Map<String, String> retMap = XmlUtil.xmlToMap(resultXml);
                String success = "SUCCESS";
                if (retMap != null && retMap.size() > 0 && success.equals(retMap.get("return_code")) && success.equals(retMap.get("result_code")))
                {
                    return retMap;
                }
            }
        }catch (Exception e)
        {
            log.error("get pay notify error.",e);
            throw  e;
        }
        return null;
    }

    /**
     * 微信回调参数验证
     * @param map
     * @return
     */
    public boolean verifyPayNotify(Map<String, String> map) {
        //商户id
        String mchId = map.get("mch_id");
        //微信支付订单号
        String payOrderId =  map.get("transaction_id");
        //商户订单号
        String mchOrderNo =  map.get("out_trade_no");
        //现金支付金额
        String amount = map.get("cash_fee");
        //签名
        String sign =  map.get("sign");

        if (StringUtil.isEmpty(mchId)) {
            log.warn("Params error. mchId : {}", mchId);
            return false;
        }
        if (StringUtil.isEmpty(payOrderId)) {
            log.warn("Params error. payOrderId : {}", payOrderId);
            return false;
        }
        if (StringUtil.isEmpty(amount)) {
            log.warn("Params error. amount : {}", amount);
            return false;
        }
        if (StringUtil.isEmpty(sign)) {
            log.warn("Params error. sign : {}", sign);
            return false;
        }

        // 验证签名
        if (!verifySign(map)) {
            log.warn("verify params sign failed. payOrderId={}", payOrderId);
            return false;
        }

        // 根据payOrderId查询业务订单,验证订单是否存在
        LambdaQueryWrapper<LymsGroupOrder> wrapper = new QueryWrapper().lambda();
        wrapper.eq(LymsGroupOrder::getOrderno, mchOrderNo);
        List<LymsGroupOrder> lymsOrders = lymsGroupOrderService.list(wrapper);
        if (CollectionUtils.isNotEmpty(lymsOrders)) {
            log.warn("local orderno not exists,payOrderId : {},orderno : {}", payOrderId, mchOrderNo);
            return false;
        }
        // 核对金额
        if (lymsOrders.get(0).getTotalPrices() != Integer.parseInt(amount)) {
            log.warn("Inconsistent payment amount ,dbPayPrice : {},payPrice : {}", lymsOrders.get(0).getTotalPrices(), amount);
            return false;
        }
        return true;
    }


    /**
     * 回调参数的签名验证
     * @param map
     * @return
     */
    public boolean verifySign(Map<String, String> map) {
        String mchId = map.get("mch_id");
        //检查商户id
        if (!Constant.MCHID.equals(mchId) ) {
            return false;
        }
        String localSign = PayDigestUtil.getSign(map, Constant.REQ_KEY, "sign");
        String sign =  map.get("sign");
        return localSign.equalsIgnoreCase(sign);
    }


    /**
     * 微信支付签名算法sign
     * @param characterEncoding
     * @param parameters
     * @return
     */
    public static String createSign(String characterEncoding,SortedMap<Object,Object> parameters){
        StringBuffer sb = new StringBuffer();
        Set es = parameters.entrySet();//所有参与传参的参数按照accsii排序（升序）
        Iterator it = es.iterator();
        while(it.hasNext()) {
            Map.Entry entry = (Map.Entry)it.next();
            String k = (String)entry.getKey();
            Object v = entry.getValue();
            if(null != v && !"".equals(v)
                    && !"sign".equals(k) && !"key".equals(k)) {
                sb.append(k + "=" + v + "&");
            }
        }

        sb.append("key=" + Constant.REQ_KEY);
        System.out.println("签名字符串:"+sb.toString());
        System.out.println("签名MD5未变大写：" + MD5Util.MD5Encode(sb.toString(), characterEncoding));
        String sign = MD5Util.MD5Encode(sb.toString(), characterEncoding).toUpperCase();
        return sign;
    }


}
