package com.lyms.talkonlineweb.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.lyms.talkonlineweb.annotation.TokenRequired;
import com.lyms.talkonlineweb.domain.*;
import com.lyms.talkonlineweb.request.PatientInfoRequest;
import com.lyms.talkonlineweb.result.BaseResponse;
import com.lyms.talkonlineweb.result.PatientPcaseResult;
import com.lyms.talkonlineweb.service.*;
import com.lyms.talkonlineweb.task.AccessTokenServlet;
import com.lyms.talkonlineweb.util.*;
import com.mysql.cj.Query;
import lombok.extern.log4j.Log4j2;
import org.apache.ibatis.annotations.Delete;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.DigestUtils;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.*;


/**
 * 患者管理
 */
@RestController
@RequestMapping("pat")
@Log4j2
public class PatientController {

    @Autowired
    private LymsPatientService lymsPatientService;//患者

    @Autowired
    private LymsPcaseService lymsPcaseService;//病例

    @Autowired
    private LymsIllnessService lymsIllnessService;//疾病

    @Autowired
    private LymsTkcardService lymsTkcardService;//问诊记录

    @Autowired
    private PatientInfoService patientInfoService;//患者视图

    @Autowired
    private LymsTcardService lymsTcardService;//问诊卡信息

    @Autowired
    private LymsLogsService lymsLogsService;//日志记录

    @Autowired
    private LymsHdepartService lymsHdepartService;//科室服务

    @Autowired
    private HXService hxService;

    //医院
    @Autowired
    private LymsHospitalService lymsHospitalService;
    //医生
    @Autowired
    private LymsDoctorService lymsDoctorService;
    //关注
    @Autowired
    private LymsAttentionService lymsAttentionService;
    @Autowired
    private LymsDictService lymsDictService;

    @Autowired   //科室所拥有的疾病
    private LymsDepartillService lymsDepartillService;
    @Autowired
    private LymsPatientAttentionService lymsPatientAttentionService;

    @Autowired
    private RegisterPatientInfoService registerPatientInfoService;
    @Autowired
    private LymsPushVisitRecordService lymsPushVisitRecordService;

    /**
     * 获取患者列表
     *
     * @param patientInfo
     * @param current
     * @param size
     * @return
     */
    @GetMapping("sltPatientLst")
    @TokenRequired
    public BaseResponse sltPatientLst(PatientInfo patientInfo, int current, int size, @RequestParam(required = false) Integer qtype) {
        BaseResponse baseResponse = new BaseResponse();
        Page<PatientInfo> page = new Page<>(current, size);
        QueryWrapper<PatientInfo> queryWrapper = Wrappers.query(patientInfo);
        if (Objects.nonNull(qtype) && qtype == 1) {//从小程序购买
            queryWrapper.inSql("id", "SELECT t.`pid` FROM lyms_tcard t WHERE t.`fid`=1");
        }
        if (Objects.nonNull(qtype) && qtype == 2) {//从医院购买
            queryWrapper.inSql("id", "SELECT t.`pid` FROM lyms_tcard t WHERE t.`fid`=2");
        }
        if(StringUtils.isEmpty(patientInfo.getPname())){
            patientInfo.setPname(null);
        }
        if(StringUtil.isNotEmpty(patientInfo.getSynthesisQuery())){
            queryWrapper.and(wrapper  -> wrapper .like("pname", patientInfo.getSynthesisQuery())
                    .or().eq("mobile", patientInfo.getSynthesisQuery())
                    .or().eq("enrolmentPhone", patientInfo.getSynthesisQuery())
                    .or().eq("idno", patientInfo.getSynthesisQuery()));
        }
        Page<PatientInfo> patientPagePage = patientInfoService.page(page, queryWrapper);
       /* List<PatientInfo> pLst = patientInfoService.list(queryWrapper);
        int daiCnt = pLst.parallelStream().mapToInt(PatientInfo::getCcnt).sum();//待使用数量

        List<Integer> pids = pLst.parallelStream().map(PatientInfo::getId).collect(Collectors.toList());

        LymsTcard tcard = new LymsTcard();
        QueryWrapper<LymsTcard> tWrapper = Wrappers.query(tcard);
        if (pids.size() == 0) {
            pids.add(-1);
        }
        tWrapper.in("pid", pids);
        int sum = lymsTcardService.list(tWrapper).parallelStream().mapToInt(LymsTcard::getCnt).sum();

        log.info("统计需要改动数据结构:daiCnt:{} sum:{}", daiCnt, sum);*/
        baseResponse.setObject(patientPagePage);

        return baseResponse;
    }

    /**
     * 保存或更新患者信息
     *
     * @param patient
     * @return
     */
    @PostMapping("savePatient")
    @TokenRequired
    public BaseResponse savePatient(@Validated LymsPatient patient, BindingResult result, LymsPcase pcase, String illness) {
        BaseResponse baseResponse = new BaseResponse();
        log.info(">>>>>>>>>>>>>>>登记病例");
        if(StringUtil.isEmpty(illness)){
            baseResponse.setErrorcode(1);
            baseResponse.setErrormsg("请选择疾病名称");
            return baseResponse;
        }

        patient.setIdno(patient.getIdno().toLowerCase());
        LymsPatient tmpP = new LymsPatient();
        tmpP.setIdno(patient.getIdno().toLowerCase());
        LymsPatient patient2 = lymsPatientService.getOne(Wrappers.query(tmpP));
        if (patient2 == null) {
            patient.setCreatedtime(new Date());
            patient.setPpasswd(DigestUtils.md5DigestAsHex(Constant.COMMON_PASSWD.getBytes()));
        } else {
            //今天有这个患者疾病的病例不用添加（这里防止同一天平台重复添加病例疾病）
            QueryWrapper<LymsPcase> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("pid", patient2.getId());
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(new Date());
            calendar.set(Calendar.HOUR_OF_DAY, 00);//时
            calendar.set(Calendar.MINUTE, 00);//分
            calendar.set(Calendar.SECOND, 00);//秒
            queryWrapper.ge("createdtime", calendar.getTime());
            List<LymsPcase> pcaseList = lymsPcaseService.list(queryWrapper);
            for (LymsPcase lymsPcase : pcaseList) {
                QueryWrapper<LymsIllness> illnessQueryWrapper = new QueryWrapper<>();
                illnessQueryWrapper.eq("pcid", lymsPcase.getPcid());
                if(StringUtil.isNotEmpty(illness)){
                    List<String> iids=Arrays.asList(illness.split(","));
                    illnessQueryWrapper.in("iid", iids);
                }

                illnessQueryWrapper.ge("createdtime", calendar.getTime());
                int count = lymsIllnessService.count(illnessQueryWrapper);
                if (count != 0) {
                    baseResponse.setErrorcode(1);
                    baseResponse.setErrormsg("今天已经添加过该患者病例。不要重复添加！");
                    return baseResponse;
                }
            }
            patient.setId(patient2.getId());
            patient.setUpdatedtime(new Date());
            patient.setCcnt(patient2.getCcnt()+  (null== patient.getCcnt()?0:patient.getCcnt()));
        }

        if (result.hasErrors()) {
            baseResponse.setErrorcode(1);
            result.getAllErrors().forEach(e -> {
                baseResponse.setErrormsg(baseResponse.getErrormsg() + e.getDefaultMessage());
            });
            baseResponse.setErrorcode(1);
            return baseResponse;
        }


        if (!StringUtils.isEmpty(patient.getCode())) {
            patient.setOpenid(WeiXinUtil.getWxOpenId(patient.getCode()));
        }
//        if (StringUtil.isEmpty(patient.getHxid())) {
//            JSONObject json = hxService.addUser(patient.getIdno(), Constant.COMMON_PASSWD, patient.getPname());
//            JSONArray rArr = json.getJSONArray("entities");
//            if (rArr.size() > 0) {
//                patient.setHxid(rArr.getJSONObject(0).getString("uuid"));
//            }
//        }

        boolean f = lymsPatientService.saveOrUpdate(patient);

//        添加病例
        pcase.setCreatedby(patient.getCreatedby());
        pcase.setPid(patient.getId());
        pcase.setCreatedtime(new Date());
        f = lymsPcaseService.saveOrUpdate(pcase);


        log.info(patient);
        log.info(pcase);

        Map param = new HashMap();
        param.put("vtype",3);
        List<LymsDict> dLst=lymsDictService.listByMap(param);
        String[] iArr = illness.split(",");

        for (int i = 0; i < iArr.length; i++) {
            LymsIllness ness = new LymsIllness();
            ness.setCreatedby(patient.getCreatedby());
            ness.setPcid(pcase.getPcid());
            ness.setIid(Integer.parseInt(iArr[i]));
            ness.setCreatedtime(new Date());

            dLst.forEach(e->{
                if(e.getCode().intValue()==ness.getIid().intValue()){
                    ness.setIname(e.getValue());
                }
            });

            param.clear();
            param.put("pcid", pcase.getPcid());
            param.put("iid", iArr[i]);
            List<LymsIllness> iLst = lymsIllnessService.listByMap(param);
            if (iLst.size() > 0) {
                ness.setId(iLst.get(0).getId());
            }
            f = lymsIllnessService.saveOrUpdate(ness);
            log.info(ness.toString());
        }

        Date instDate = new Date();
        //需要添加医院购买卡记录
        for (int i = 0; i < patient.getCcnt(); i++) {
            LymsTcard tcard = new LymsTcard();
            tcard.setCnt(1);
            tcard.setPcid(pcase.getPcid());
            tcard.setPid(patient.getId());
            tcard.setFid(2);
            tcard.setCreatedby(patient.getCreatedby());
            tcard.setCreatedtime(instDate);

            lymsTcardService.saveOrUpdate(tcard);
        }
        baseResponse.setObject(patient);
        baseResponse.setErrorcode(0);
        return baseResponse;
    }

    /**
     * 更新患者信息---小程序端使用
     *
     * @param patient
     * @return
     */
    @PostMapping("updatePatient")
    @TokenRequired
    public BaseResponse updatePatient(@RequestBody LymsPatient patient){
        BaseResponse baseResponse=new BaseResponse();
        boolean f=false;
        if(Objects.nonNull(patient.getId()) ){
            patient.setUpdatedtime(new Date());
            if(StringUtil.isNotEmpty(patient.getPpasswd())) {
                patient.setPpasswd(DigestUtils.md5DigestAsHex(patient.getPpasswd().getBytes()));
            }
            f=lymsPatientService.saveOrUpdate(patient);
        }
        baseResponse.setErrorcode(f==true?0:1);
        return baseResponse;
    }

    /**
     * 删除患者
     *
     * @param id
     * @return
     */
    @GetMapping("delPatient")
    @TokenRequired
    public BaseResponse delPatient(int id) {
        BaseResponse baseResponse = new BaseResponse();
        boolean f = lymsPatientService.removeById(id);
        if (f){
            final LymsPatient patient = lymsPatientService.getById(id);
            if (StringUtil.isNotEmpty(patient.getHxid())) {
                JSONObject rs=hxService.delUser(patient.getIdno());
                log.info(rs);
            }

        }
        baseResponse.setErrorcode(f == true ? 0 : 1);
        return baseResponse;
    }

    /**
     * 删除患者病例ID
     *
     * @param pcid
     * @return
     */
    @GetMapping("delPcase")
    @TokenRequired
    public BaseResponse delPcase(int pcid) {
        BaseResponse baseResponse = new BaseResponse();
        boolean f = lymsPcaseService.removeById(pcid);
        baseResponse.setErrorcode(f == true ? 0 : 1);
        return baseResponse;
    }

    /**
     * 患者登录
     *
     * @param patient
     * @return
     */
    @PostMapping("loginPatient")
    public BaseResponse loginPatient(@RequestBody LymsPatient patient) {
        BaseResponse baseResponse = new BaseResponse();

        //公众号跳转小程序登录状态。loginType==1不需要加密处理,这里是反相判断
        if(!((Integer)1).equals(patient.getLoginType())){
            //密码加密
            patient.setPpasswd(DigestUtils.md5DigestAsHex(patient.getPpasswd().getBytes()));
        }
        List<LymsPatient> dLst = lymsPatientService.list(Wrappers.query(patient));
        baseResponse.setErrorcode(1);
        LymsPatient patient2=null;

        //是否有病例
        int pcaseSize=0;
        LymsPcase pcase=new LymsPcase();
        pcase.setMobile(patient.getIdno());
        List<LymsPcase> cLst =lymsPcaseService.list(Wrappers.query(pcase));
        pcaseSize=cLst.size();
        if(dLst.size() < 1 ){//手机号登录
            if(pcaseSize>0){
                Map<String,Object> param=new HashMap<>();
                param.put("id",cLst.get(0).getPid());
                param.put("ppasswd",patient.getPpasswd());
                dLst=lymsPatientService.listByMap(param);
            }else {//没有病例
                //小程序自主注册用手机号登录验证
                Map<String,Object> paramQuery=new HashMap<>();
                paramQuery.put("enrolment_phone",patient.getIdno());
                paramQuery.put("ppasswd",patient.getPpasswd());
                if(patient.getId()!=null) {
                    paramQuery.put("id", patient.getId());
                }
                List<LymsPatient> patients = lymsPatientService.listByMap(paramQuery);
                if(patients.size()==1) {
                    dLst= patients;
                }
                if(patients.size()>1){//一个手机号码注册多个身份证
                    List<Map> listMap=new ArrayList<>();
                    for (LymsPatient lymsPatient : patients) {
                        Map map=new HashMap();
                        map.put("name", lymsPatient.getPname());
                        map.put("id", lymsPatient.getId());
                        listMap.add(map);
                    }
                    baseResponse.setErrorcode(2);
                    baseResponse.setObject(listMap);
                }
            }

        }


        if (dLst.size() > 0) {
            patient2 = dLst.get(0);

            String jwt = JwtUtils.createJWT("1", patient.getIdno(), Constant.JWT_TTL);
            Map<String, Object> map = new HashMap<>();

            patient2.setIslogin(1);

            if (StringUtils.isEmpty(patient2.getHxid()) || StringUtils.isEmpty(patient2.getOpenid())) {

//                if (patient2.getHxid() == null) {
//                    JSONObject json = hxService.addUser(patient2.getIdno(), Constant.COMMON_PASSWD, patient2.getPname());
//                    JSONArray rArr = json.getJSONArray("entities");
//                    if (rArr.size() > 0) {
//                        patient2.setHxid(rArr.getJSONObject(0).getString("uuid"));
//                    }
//                }
                if (StringUtils.isEmpty(patient2.getOpenid())) {
                    patient2.setOpenid(WeiXinUtil.getWxOpenId(patient.getCode()));
                }
                lymsPatientService.saveOrUpdate(patient2);
            }

            patient.setPpasswd(null);
            map.put("patient", patient2);
            map.put("token", jwt);
            map.put("pcaseSize", pcaseSize);
            baseResponse.setErrorcode(0);
            baseResponse.setObject(map);
        }
        return baseResponse;
    }

    /**
     * 查询单个患者信息
     *
     * @param patientInfo
     * @return
     */
    @GetMapping("queryPatient")
    @TokenRequired
    public BaseResponse queryPatient(PatientInfo patientInfo) {
        BaseResponse baseResponse = new BaseResponse();
        PatientInfo patientInfo2 = patientInfoService.getOne(Wrappers.query(patientInfo));
        baseResponse.setObject(patientInfo2);
        return baseResponse;
    }
    /**
     * 查询患者信息所有病例
     *
     * @param patientInfo
     * @return
     */
    @GetMapping("queryPatient2")
    @TokenRequired
    public BaseResponse queryPatient2(PatientInfo patientInfo) {
        BaseResponse baseResponse = new BaseResponse();
        List<PatientInfo> patientInfo2 = patientInfoService.list(Wrappers.query(patientInfo));
        baseResponse.setObject(patientInfo2);
        return baseResponse;
    }


    /**
     * 传入患者id，获取病历对应的医院列表
     *
     * @param lymsPcase
     * @return
     */
    @GetMapping("getPatientHospitals")
    @TokenRequired
    public BaseResponse getPatientHospitals(LymsPcase lymsPcase) {
        BaseResponse baseResponse = new BaseResponse();
        List<LymsPcase> pcases = lymsPcaseService.list(Wrappers.query(lymsPcase));
        baseResponse.setObject(pcases);
        return baseResponse;
    }


    /**
     * 查询患者在某个医院的详细诊断病历
     * 需要传入患者id和医院id
     *
     * @param pid 患者id
     * @param hid 医院id
     * @return
     */
    @GetMapping("getPatientHospitalPcase")
    @TokenRequired
    public BaseResponse getPatientHospitalPcase(int pid, int hid,
                                                @RequestParam(required = false) String keyword) {
        BaseResponse baseResponse = new BaseResponse();
        Map data = new HashMap(5);
        LymsPatient patient = lymsPatientService.getById(pid);
        //患者基本信息
        data.put("patient", patient);

        //医院信息
        LymsHospital hospital = lymsHospitalService.getById(pid);
        data.put("hospital", hospital);

        List<Integer> ilist = new ArrayList<>();
        if (StringUtil.isNotEmpty(keyword)) {
            LambdaQueryWrapper<LymsIllness> iwrapper = new QueryWrapper().lambda();
            iwrapper.like(LymsIllness::getIname, keyword);
            List<LymsIllness> illnesses = lymsIllnessService.list(iwrapper);
            if (CollectionUtils.isNotEmpty(illnesses)) {
                illnesses.stream().forEach(i -> {
                    ilist.add(i.getPcid());
                });
            }
        }
        //查询患者的就诊记录
        LambdaQueryWrapper<LymsPcase> wrapper = new QueryWrapper().lambda();
        wrapper.eq(LymsPcase::getPid, pid);
        wrapper.eq(LymsPcase::getHid, hid);
        if (StringUtil.isNotEmpty(keyword)) {
            wrapper.and(i -> i.like(LymsPcase::getDname, keyword).or().like(LymsPcase::getDtname, keyword));
            if (CollectionUtils.isNotEmpty(ilist)) {
                wrapper.or(i -> i.in(LymsPcase::getPcid, ilist));
            }
        }
        wrapper.orderByDesc(LymsPcase::getCreatedtime);
        List<LymsPcase> lymsPcaseList = lymsPcaseService.list(wrapper);

        List<PatientPcaseResult> pcases = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(lymsPcaseList)) {
            lymsPcaseList.forEach(destModel -> {
                PatientPcaseResult result = new PatientPcaseResult();
                result.setPid(destModel.getPid());
                result.setDepartName(destModel.getDname());
                result.setCardNo(patient.getIdno());
                result.setPatientName(patient.getPname());
                result.setCcnt(patient.getCcnt());
                result.setPhxid(patient.getHxid());
                LymsDoctor doctor = lymsDoctorService.getById(destModel.getDtid());
                //获得医生职位
                result.setDoctorName(doctor.getDname());
                LambdaQueryWrapper<LymsDict> wrapperLymsDict = new QueryWrapper().lambda();
                wrapperLymsDict.eq(LymsDict::getVtype, 4);
                wrapperLymsDict.eq(LymsDict::getCode, doctor.getLvl());
                LymsDict dict=lymsDictService.getOne(wrapperLymsDict);
                result.setDoctorLvl(dict.getValue());
                //获得疾病名称
                LambdaQueryWrapper<LymsIllness> wrapperLymsIllness = new QueryWrapper().lambda();
                wrapperLymsIllness.eq(LymsIllness::getPcid,destModel.getPcid());
                List<LymsIllness> lymsIllnessList=lymsIllnessService.list(wrapperLymsIllness);
                String inames="";
                for (LymsIllness lymsIllness : lymsIllnessList) {
                    LambdaQueryWrapper<LymsDict> wrapperLymsDict2 = new QueryWrapper().lambda();
                    wrapperLymsDict2.eq(LymsDict::getVtype, 3);
                    wrapperLymsDict2.eq(LymsDict::getCode,lymsIllness.getIid());
                    LymsDict dict2=lymsDictService.getOne(wrapperLymsDict2);
                    inames+=dict2.getValue()+",";
                }
                result.setIname(StringUtil.isNotEmpty(inames)?inames.substring(0, inames.length()-1):"");
                //获得医生环信账号
                result.setDlogin(doctor.getDlogin());
                result.setDhxid(doctor.getHxid());
                result.setDtid(destModel.getDtid());
                result.setCreatedby(DateUtil.getYyyyMmDdHhMmSs(destModel.getCreatedtime()));
                //病例id
                result.setPcid(destModel.getPcid());
                pcases.add(result);
            });
        }
        data.put("pcases", pcases);

        baseResponse.setObject(data);
        return baseResponse;
    }


    /**
     * 通过患者id查询患者的基本信息
     *
     * @param lymsPatient
     * @return
     */
    @GetMapping("getPatientBaseInfo")
    @TokenRequired
    public BaseResponse getPatientBaseInfo(LymsPatient lymsPatient) {

        BaseResponse baseResponse = new BaseResponse();

        Map data = new HashMap(6);
        LymsPatient patients = lymsPatientService.getById(lymsPatient.getId());

        //查询患者的就诊记录
        LambdaQueryWrapper<LymsPcase> wrapper = new QueryWrapper().lambda();
        wrapper.eq(LymsPcase::getPid, lymsPatient.getId());
        wrapper.orderByDesc(LymsPcase::getCreatedtime);
        List<LymsPcase> lymsPcaseList = lymsPcaseService.list(wrapper);

        data.put("pid", patients.getId());
        data.put("pname", patients.getPname());
        data.put("psex", patients.getSex());
        data.put("birth", patients.getBirth());
        if (CollectionUtils.isNotEmpty(lymsPcaseList)) {
            data.put("mobile", lymsPcaseList.get(0).getMobile());
        }
        baseResponse.setObject(data);
        return baseResponse;
    }


    /**
     * 更新患者信息
     *
     * @param request
     * @return
     */
    @PutMapping("updatePatientInfo")
    @TokenRequired
    public BaseResponse updatePatientInfo(@RequestBody PatientInfoRequest request) {
        BaseResponse baseResponse = new BaseResponse();
        LymsPatient patients = lymsPatientService.getById(request.getPid());
        patients.setBirth(request.getBirth());
        patients.setPname(request.getPname());
        patients.setSex(request.getPsex());
        lymsPatientService.updateById(patients);

        //查询患者的就诊记录
        LambdaQueryWrapper<LymsPcase> wrapper = new QueryWrapper().lambda();
        wrapper.eq(LymsPcase::getPid, request.getPid());
        wrapper.orderByDesc(LymsPcase::getCreatedtime);
        List<LymsPcase> lymsPcaseList = lymsPcaseService.list(wrapper);
        if (CollectionUtils.isNotEmpty(lymsPcaseList)) {
            lymsPcaseList.forEach(lymsPcase -> {
                lymsPcase.setMobile(request.getMobile());
                lymsPcaseService.updateById(lymsPcase);
            });
        }
        return baseResponse;
    }


    /**
     * 患者的问诊记录接口
     *
     * @param lymsTkrecord 传入患者id参数
     * @return
     */
    @GetMapping("getPatienttkrecord")
    @TokenRequired
    public BaseResponse getPatienttkrecord(LymsTkrecord lymsTkrecord) {

        BaseResponse baseResponse = new BaseResponse();
        LymsPatient patients = lymsPatientService.getById(lymsTkrecord.getPid());

        //查询患者的就诊记录
        LambdaQueryWrapper<LymsTkrecord> wrapper = new QueryWrapper().lambda();
        wrapper.eq(LymsTkrecord::getPid, lymsTkrecord.getPid());
        wrapper.orderByDesc(LymsTkrecord::getCreatedtime);
        List<LymsTkrecord> lymsTkrecords = lymsTkcardService.list(wrapper);
        Map param=new HashMap();
        param.put("vtype",4);
        List<LymsDict> dLst=lymsDictService.listByMap(param);

        List<Map> list = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(lymsTkrecords)) {
            for (LymsTkrecord record : lymsTkrecords) {
                LymsPcase pcase = lymsPcaseService.getById(record.getPcid());
                LymsHospital hospital = lymsHospitalService.getById(pcase.getHid());
                LymsDoctor doctor = lymsDoctorService.getById(pcase.getDtid());

                Map data = new HashMap(20);
                data.put("hospitalName", pcase.getHname());
                data.put("hospitalLevel", hospital.getHlevel());
                data.put("doctorName", pcase.getDtname());
                data.put("departName", pcase.getDname());

                data.put("patientName", patients.getPname());

                dLst.forEach(dd->{
                    if(dd.getCode()==doctor.getLvl()){
                        data.put("doctorLevel", dd.getValue());
                    }
                });

                StringBuilder sb = new StringBuilder();
                LambdaQueryWrapper<LymsIllness> iwrapper = new QueryWrapper().lambda();
                iwrapper.like(LymsIllness::getPcid, pcase.getPcid());
                List<LymsIllness> illnesses = lymsIllnessService.list(iwrapper);
                if (CollectionUtils.isNotEmpty(illnesses)) {
                    illnesses.forEach(lymsIllness -> {
                        sb.append(lymsIllness.getIname());
                        sb.append("  ");
                    });
                }

                data.put("iname", sb.toString());
                data.put("createTime", DateUtil.getDateTime(record.getCreatedtime(), DateUtil.YYYY_MM_DD));
                data.put("stat", record.getStat());
                list.add(data);
            }
        }
        baseResponse.setObject(list);
        return baseResponse;
    }

    /**
     * 查询患者关注的医生记录
     *
     * @param lymsAttention 传入患者id参数
     * @return
     */
    @GetMapping("getPatientAttentions")
    @TokenRequired
    public BaseResponse getPatientAttentions(LymsAttention lymsAttention) {

        BaseResponse baseResponse = new BaseResponse();
        //查询患者的关注医生记录
        LambdaQueryWrapper<LymsAttention> wrapper = new QueryWrapper().lambda();
        wrapper.eq(LymsAttention::getPid, lymsAttention.getPid());
        wrapper.orderByDesc(LymsAttention::getCreatedtime);
        List<LymsAttention> lymsAttentions = lymsAttentionService.list(wrapper);
        Map param=new HashMap();
        param.put("vtype",4);
        List<LymsDict> dLst=lymsDictService.listByMap(param);

        List<Map> list = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(lymsAttentions)) {
            for (LymsAttention attention : lymsAttentions) {
                LymsDoctor doctor = lymsDoctorService.getById(attention.getDid());
                LymsHdepart depart = lymsHdepartService.getById(doctor.getDpid());
                LymsHospital hospital = lymsHospitalService.getById(doctor.getHid());
                Map data = new HashMap(20);
                data.put("doctorName", doctor.getDname());
                data.put("doctorSex", doctor.getSex());
                data.put("departName", depart.getDname());
                data.put("did", attention.getDid());
                data.put("doctorImage", doctor.getHeadimg());
                data.put("hospitalName", depart.getHname());
                data.put("hospitalLevel", hospital.getHlevel());
                data.put("doctorDesc", doctor.getIntro());
                dLst.forEach(dd->{
                    if(dd.getCode()==doctor.getLvl()){
                        data.put("doctorLevel", dd.getValue());
                    }
                });
                list.add(data);
            }
        }
        baseResponse.setObject(list);
        return baseResponse;
    }
    /**
     * 小程序用户端-登录后显示用户病历所在医院
     *
     * @param patientId 传入患者id
     * @return 医院id和医院名称
     */
    @GetMapping("getAppPatientHospital")
    @TokenRequired
    public BaseResponse getAppPatientHospital(Integer patientId) {
        BaseResponse baseResponse = new BaseResponse();
        List<Map<String,String>>patientListMap=lymsPatientService.getAppPatientHospital(patientId);
        baseResponse.setObject(patientListMap);
        return baseResponse;
    }

    /**
     * 更新患者待用卡数量---小程序端使用
     *
     * @param patient 患者ID
     * @return
     */
    @GetMapping("updateCcnt")
    @TokenRequired
    public BaseResponse updateCcnt(LymsPatient patient){
        BaseResponse baseResponse=new BaseResponse();
        boolean f=false;
        LymsPatient patient2=lymsPatientService.getById(patient.getId());
        if(Objects.nonNull(patient2)){//验证问诊卡次数
            int cnt=patient2.getCcnt()-1;
            if (cnt < 0) {
                baseResponse.setErrorcode(1);
                baseResponse.setErrormsg("问诊卡次数错误");
                return baseResponse;
            }else{
                patient.setCcnt(cnt);
            }
        }
        if(Objects.nonNull(patient.getId()) ){
            patient.setUpdatedtime(new Date());
            f=lymsPatientService.saveOrUpdate(patient);
        }
        baseResponse.setErrorcode(f==true?0:1);
        return baseResponse;
    }

    /**
     * 患者统计待使用和总量问诊卡信息
     * @return
     */
    @GetMapping("cardStat")
    @TokenRequired
    public BaseResponse cardStat(Integer type){
        BaseResponse baseResponse = new BaseResponse();
        String pSql="";
        String cSql="";
        if (type != null) {
            pSql="WHERE id IN (SELECT c.`pid` FROM lyms_tcard c WHERE c.`fid`="+type+")";
            cSql="WHERE fid="+type;
        }
        Map<String,Object> rs= lymsTcardService.cardStat(pSql,cSql);
        baseResponse.setObject(rs);
        return baseResponse;
    }
    /**
     * 小程序-用户关注医生保存关注记录
     * @return
     */
    @PostMapping("saveAttention")
    @TokenRequired
    public BaseResponse saveAttention(@RequestBody LymsAttention lymsAttention){
        BaseResponse baseResponse=new BaseResponse();
        try {
            lymsAttentionService.save(lymsAttention);
            baseResponse.setErrormsg("成功");
        } catch (Exception e) {
            baseResponse.setErrormsg("失败");
            e.printStackTrace();
        }
        return baseResponse;
    }
    /**
     * 小程序-查询用户是否关注该医生
     * @return true 可以关注 false 已关注不能再关注
     */
    @GetMapping("getDoctorById")
    @TokenRequired
    public BaseResponse getDoctorById(LymsAttention lymsAttention){
        BaseResponse baseResponse=new BaseResponse();
        try {
            LymsAttention attention=lymsAttentionService.getOne(Wrappers.query(lymsAttention));
            baseResponse.setObject(null==attention?true:false);
            baseResponse.setErrormsg("成功");
        } catch (Exception e) {
            baseResponse.setErrormsg("失败");
            e.printStackTrace();
        }
        return baseResponse;
    }

    /**
     * PC-二次添加患者，输入身份证号显示相关信息
     * @param idno 患者身份证号码
     * @return
     */
    @GetMapping("getPatientByIdno")
    @TokenRequired
    public BaseResponse getPatientByIdno(String idno){
        BaseResponse baseResponse=new BaseResponse();
        try {
            List<PatientInfo> pLst=patientInfoService.list(new QueryWrapper<PatientInfo>().eq("idno", idno));

            //处理自主建档的情况（没有病例）要显示患者基本信息
            LymsPatient patient = lymsPatientService.getOne(new QueryWrapper<LymsPatient>().eq("idno", idno));
            if(null!=patient && CollectionUtils.isNotEmpty(pLst)){
                baseResponse.setObject(pLst.get(0));
            }
            if(CollectionUtils.isEmpty(pLst)&&null!=patient){
                baseResponse.setObject(patient);
            }
            baseResponse.setErrormsg("成功");
        } catch (Exception e) {
            baseResponse.setErrormsg("失败");
            e.printStackTrace();
        }
        return baseResponse;
    }

    /**
     * 记录患者登录日志，包括切换医院
     * @param logEntity
     */
    @GetMapping("saveLoginLog")
    @TokenRequired
    public void saveLoginLog(LymsLogs logEntity ){

        log.debug(">>>>>>>>>"+logEntity);
        logEntity.setLogDate(new Date());
        lymsLogsService.save(logEntity);
    }

    @PostMapping("updatePatientCase")
    @TokenRequired
    public BaseResponse updatePatient(LymsPatient patient,LymsPcase pcase,LymsIllness illness,Integer ilid){
        BaseResponse baseResponse=new BaseResponse();
        baseResponse.setErrorcode(1);
        baseResponse.setErrormsg("修改失败!");
        boolean f=false;
        if(null!=patient.getId() && null!=pcase.getPcid() && null!=illness.getId()){

            f=lymsPatientService.saveOrUpdate(patient);
            f=lymsPcaseService.saveOrUpdate(pcase);
            illness.setId(ilid);//前端传入一组夸对象组合。后端用3组对象接受。前端传的是ilid这里id值是错误的。需要ilid赋给id。
            f=lymsIllnessService.saveOrUpdate(illness);
            if(f){
                baseResponse.setErrorcode(0);
                baseResponse.setErrormsg("成功");
            }
        }

        return baseResponse;
    }

    /**
     * 选科室后查询出科室下有哪些疾病
     * @param did
     * @return
     */
    @GetMapping("getDepartIllness")
    @TokenRequired
    public BaseResponse getDepartIllness(String did){
        BaseResponse baseResponse=new BaseResponse();
        try {
            Map param=new HashMap();
            param.put("did",did);
            List<LymsDepartill> iLst=lymsDepartillService.listByMap(param);
            List<LymsDict> dictList=new ArrayList<>();
            for (LymsDepartill lymsDepartill : iLst) {
                final LymsDict dict = lymsDictService.getById(lymsDepartill.getIid());
                if(null!=dict){
                    dictList.add(dict);
                }
            }
            baseResponse.setObject(dictList);
            baseResponse.setErrormsg("成功");
        } catch (Exception e) {
            baseResponse.setErrormsg("失败");
            e.printStackTrace();
        }
        return baseResponse;
    }

    /**
     * 修改患者密码/重置密码
     * @param patient
     * @return
     */
    @PostMapping("updatePatientPassword")
    @TokenRequired
    public BaseResponse updatePatientPassword(@RequestBody LymsPatient patient){
        BaseResponse baseResponse=new BaseResponse();
        if(1==patient.getType()){//1 重置密码
            patient.setPpasswd(DigestUtils.md5DigestAsHex(Constant.COMMON_PASSWD.getBytes()));
        }else{//修改
            patient.setPpasswd(DigestUtils.md5DigestAsHex(patient.getPpasswd().getBytes()));
        }
        boolean f=lymsPatientService.updateById(patient);
        baseResponse.setErrorcode(f==true?0:1);
        baseResponse.setErrormsg(f==true?"成功":"失败");
        return baseResponse;
    }

    /**
     * 小程序注册患者
     *
     * @param patient
     * @return
     */
    @PostMapping("signInPatient")
//    @TokenRequired
    public BaseResponse signInPatient(@RequestBody LymsPatient patient) {
        BaseResponse baseResponse = new BaseResponse();
        patient.setIdno(patient.getIdno().toLowerCase());
        LymsPatient tmpP=new LymsPatient();
        tmpP.setIdno(patient.getIdno().toLowerCase());
        LymsPatient patient2 = lymsPatientService.getOne(Wrappers.query(tmpP).eq("idno", patient.getIdno()));
        if (patient2 == null) {
            patient.setCreatedtime(new Date());
            if(StringUtil.isNotEmpty(patient.getPpasswd())){
                patient.setPpasswd(DigestUtils.md5DigestAsHex(patient.getPpasswd().getBytes()));
            }
            boolean f = lymsPatientService.save(patient);
            baseResponse.setErrorcode(f?0:1);
            baseResponse.setErrormsg(f?"注册成功！":"注册失败！");
        }else {
            baseResponse.setErrorcode(2);
            baseResponse.setErrormsg("身份证已注册！");
        }
        return baseResponse;
    }
    /**
     * 小程序关注患者查询or是否关注过
     *
     * @param patient
     * @return
     */
    @GetMapping("getPatientAttention")
    @TokenRequired
    public BaseResponse getPatientAttention(LymsPatient patient) {
        List<Map> result=new ArrayList<>();
        if(null!=patient.getId()) {
            QueryWrapper<LymsPatient> queryWrapper = new QueryWrapper<>();
            if (StringUtil.isNotEmpty(patient.getIdno())) {
                queryWrapper.eq("p.idno", patient.getIdno());
            }
            if (StringUtil.isNotEmpty(patient.getEnrolmentPhone())) {
                queryWrapper.eq("p.enrolment_phone", patient.getEnrolmentPhone())
                        .or().eq("pc.mobile", patient.getEnrolmentPhone());
            }
            //指定返回列
            queryWrapper.lambda().select(LymsPatient::getId,
                    LymsPatient::getPname,
                    LymsPatient::getSex,
                    LymsPatient::getBirth,
                    LymsPatient::getIdno,
                    LymsPatient::getCreatedtime);
            result = lymsPatientService.getPatient(queryWrapper);
            if (CollectionUtils.isNotEmpty(result)) {
                Iterator<Map> iterator = result.iterator();
                while (iterator .hasNext()) {
                    Map map= iterator.next();
                    if(map.get("id").equals(patient.getId())){
                        iterator.remove();
                        continue;
                    }
                    QueryWrapper<LymsPatientAttention> queryWrapper2 = new QueryWrapper<>();
                    queryWrapper2.lambda().eq(LymsPatientAttention::getPid, patient.getId());
                    queryWrapper2.lambda().eq(LymsPatientAttention::getAttentionId, map.get("id"));
                    final int count = lymsPatientAttentionService.count(queryWrapper2);
                    if (count == 0) {
                        map.put("type", 2);//未关注
                    } else {
                        map.put("type", 1);//已关注
                    }
                }
            }
        }

        BaseResponse baseResponse = new BaseResponse();
        baseResponse.setErrorcode(0);
        baseResponse.setObject(result);
        baseResponse.setErrormsg("成功");
        return baseResponse;
    }

    /**
     * 小程序关注的患者列表
     *
     * @param patientAttention
     * @return
     */
    @GetMapping("getPatientAttentionList")
    @TokenRequired
    public BaseResponse getPatientAttentionList(LymsPatientAttention patientAttention) {
        BaseResponse baseResponse = new BaseResponse();
        if (null==patientAttention.getPid()) {
            baseResponse.setErrorcode(1);
            baseResponse.setErrormsg("参数错误");
            return baseResponse;
        }
        QueryWrapper<LymsPatientAttention> queryWrapper=new QueryWrapper<>();
        if(null!=patientAttention.getPid()){
            queryWrapper.eq("pa.pid", patientAttention.getPid());
        }
        List<Map> resultList=lymsPatientAttentionService.getPatientList(queryWrapper);
        baseResponse.setErrorcode(0);
        baseResponse.setObject(resultList);
        baseResponse.setErrormsg("成功");
        return baseResponse;
    }
    /**
     * 小程序关注患者
     *
     * @param
     * @return
     */
    @PostMapping("addPatientAttention")
    @TokenRequired
    public BaseResponse addPatientAttention(@RequestBody @Validated LymsPatientAttention patientAttention,BindingResult bindingResult) {
        BaseResponse baseResponse = new BaseResponse();
        baseResponse.setErrorcode(0);
        baseResponse.setErrormsg("成功");
        //对象验证
        if(bindingResult.hasErrors()){
            baseResponse.setErrorcode(1);
            for (ObjectError allError : bindingResult.getAllErrors()) {
                baseResponse.setErrormsg((StringUtil.isNotEmpty(baseResponse.getErrormsg())?baseResponse.getErrormsg():"")+allError.getDefaultMessage()+";");
            }
            return baseResponse;
        }
        final boolean result = lymsPatientAttentionService.save(patientAttention);
        if(!result){
            baseResponse.setErrorcode(1);
            baseResponse.setErrormsg("保存异常！");
        }
        return baseResponse;
    }
    /**
     * 查询患者是否关注了公众号
     *
     * @param gzopengId
     * @return
     */
    @GetMapping("getPatientAttentionYn")
    @TokenRequired
    public BaseResponse getPatientAttentionYn(String gzopengId) {
        BaseResponse baseResponse = new BaseResponse();
        if (StringUtil.isEmpty(gzopengId)) {
            baseResponse.setErrorcode(1);
            baseResponse.setErrormsg("参数错误");
            return baseResponse;
        }
        if (StringUtil.isEmpty(AccessTokenServlet.accessToken)) {
            baseResponse.setErrorcode(1);
            baseResponse.setErrormsg("Token错误");
            return baseResponse;
        }
        String result = HttpUtil.getData("https://api.weixin.qq.com/cgi-bin/user/info?access_token="+AccessTokenServlet.accessToken+"&openid="+gzopengId+"&lang=zh_CN");
        Map<String, Object> resultMap = JsonUtil.str2Map(result, HashMap.class);
        baseResponse.setErrorcode(0);
        baseResponse.setObject(resultMap);
        baseResponse.setErrormsg("成功");
        return baseResponse;
    }

    /**
     * 小程序取消关注患者
     *
     * @param patientAttention
     * @return
     */
    @PostMapping("cancelPatientAttention")
    @TokenRequired
    public BaseResponse cancelPatientAttention(@RequestBody @Validated LymsPatientAttention patientAttention,BindingResult bindingResult) {
        BaseResponse baseResponse = new BaseResponse();
        baseResponse.setErrorcode(0);
        baseResponse.setErrormsg("成功");
        //对象验证
        if(bindingResult.hasErrors()){
            baseResponse.setErrorcode(1);
            for (ObjectError allError : bindingResult.getAllErrors()) {
                baseResponse.setErrormsg((StringUtil.isNotEmpty(baseResponse.getErrormsg())?baseResponse.getErrormsg():"")+allError.getDefaultMessage()+";");
            }
            return baseResponse;
        }
        QueryWrapper<LymsPatientAttention> queryWrapper=new QueryWrapper<>();
        queryWrapper.lambda().eq(LymsPatientAttention::getPid, patientAttention.getPid());
        queryWrapper.lambda().eq(LymsPatientAttention::getAttentionId, patientAttention.getAttentionId());
        final boolean result = lymsPatientAttentionService.remove(queryWrapper);
        if(!result){
            baseResponse.setErrorcode(1);
            baseResponse.setErrormsg("保存异常！");
        }
        return baseResponse;
    }

    /**
     * PC-注册的患者列表（没有病例的患者）
     *
     * @param registerPatientInfo
     * @param current
     * @param size
     * @return
     */
    @GetMapping("getRegisterPatientInfo")
    @TokenRequired
    public BaseResponse getRegisterPatientInfo(RegisterPatientInfo registerPatientInfo, int current, int size) {
        BaseResponse baseResponse = new BaseResponse();
        try {
            QueryWrapper<RegisterPatientInfo> query=new QueryWrapper();
            if(StringUtil.isNotEmpty(registerPatientInfo.getSynthesisQuery())){
                query.and(wrapper  -> wrapper .like("pname", registerPatientInfo.getSynthesisQuery())
                        .or().eq("enrolment_phone", registerPatientInfo.getSynthesisQuery())
                        .or().eq("idno", registerPatientInfo.getSynthesisQuery()));
            }
            if(StringUtil.isNotEmpty(registerPatientInfo.getStartCreatedtime())){
                query.ge("createdtime", registerPatientInfo.getStartCreatedtime());
            }
            if(StringUtil.isNotEmpty(registerPatientInfo.getEndCreatedtime())){
                query.le("createdtime", registerPatientInfo.getEndCreatedtime());
            }
            query.orderByDesc("createdtime");
            Page<RegisterPatientInfo> page= new Page<>(current,size);
            Page<RegisterPatientInfo> registerPatientInfoPage = registerPatientInfoService.page(page,query);
            baseResponse.setObject(registerPatientInfoPage);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return baseResponse;
    }

    /**
     *  PC-查看医生回访推送公众号记录
     * @param visitRecord
     * @param current
     * @param size
     * @return
     */
    @GetMapping("getPushVisitRecord")
    @TokenRequired
    public BaseResponse getPushVisitRecord(LymsPushVisitRecord visitRecord, int current, int size) {
        BaseResponse baseResponse = new BaseResponse();
        try {
            QueryWrapper query=new QueryWrapper();
            if(StringUtil.isNotEmpty(visitRecord.getStratTime())&&StringUtil.isNotEmpty(visitRecord.getEndTime())){
                query.apply("date_format(lpvr.createdtime,'%Y-%m-%d')>=date_format({0},'%Y-%m-%d')", visitRecord.getStratTime());
                query.apply("date_format(lpvr.createdtime,'%Y-%m-%d')<=date_format({0},'%Y-%m-%d')", visitRecord.getEndTime());
            }
            if (null!=visitRecord.getHid()) {
                query.eq("d.hid", visitRecord.getHid());
            }
            if (null!=visitRecord.getDpid()) {
                query.eq("d.dpid", visitRecord.getDpid());
            }
            if (StringUtil.isNotEmpty(visitRecord.getPname())) {
                query.like("lpvr.pname", visitRecord.getPname());
            }
            if (StringUtil.isNotEmpty(visitRecord.getDname())) {
                query.like("lpvr.dname", visitRecord.getDname());
            }
            if (null!=visitRecord.getIid()) {
                query.eq("lpvr.iid", visitRecord.getIid());
            }
            if (null!=visitRecord.getState()) {
                query.eq("lpvr.state", visitRecord.getState());
            }
            if (null!=visitRecord.getType()) {
                query.eq("lpvr.type", visitRecord.getType());
            }
            query.orderByDesc("lpvr.createdtime");
            Page<Map<String,Object>> page= new Page<>(current,size);
            List<Map<String,Object>> visitRecordList = lymsPushVisitRecordService.getPage(page,query);
            page.setRecords(visitRecordList);
            baseResponse.setObject(page);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return baseResponse;
    }
    /**
     * PC新模块建档/修改患者
     *
     * @param patient
     * @return
     */
    @PostMapping("addOrUpdateBookbuildingPatient")
    @TokenRequired
    public BaseResponse bookbuildingPatient(@RequestBody LymsPatient patient) {
        BaseResponse baseResponse = new BaseResponse();
        try {
            QueryWrapper<LymsPatient> queryWrapper=new QueryWrapper<>();
            queryWrapper.lambda().eq(LymsPatient::getIdno, patient.getIdno());
            List<LymsPatient> patientList = lymsPatientService.list(queryWrapper);
            if(null==patient.getId()) {
                if (0<patientList.size()) {
                    baseResponse.setErrorcode(1);
                    baseResponse.setErrormsg("身份证已注册，请查询档案信息。");
                    return baseResponse;
                }
                patient.setPpasswd(DigestUtils.md5DigestAsHex("123456".getBytes()));
            }else {
                LymsPatient lymsPatient = lymsPatientService.getById(patient.getId());
                if(0<patientList.size() && ! patientList.get(0).getIdno().equals(lymsPatient.getIdno())){
                    baseResponse.setErrorcode(1);
                    baseResponse.setErrormsg("身份证已注册，请查询档案信息。");
                    return baseResponse;
                }
            }
            boolean f = lymsPatientService.saveOrUpdate(patient);
            baseResponse.setErrorcode(f?0:1);
            baseResponse.setErrormsg(f?(null==patient.getId()?"注册成功！":"修改成功"):(null==patient.getId()?"注册失败！":"修改失败"));
        } catch (Exception e) {
            baseResponse.setErrorcode(1);
            baseResponse.setErrormsg("系统异常");
            e.printStackTrace();
        }
        return baseResponse;
    }
    /**
     * PC新模块删除患者
     *
     * @param id
     * @return
     */
    @DeleteMapping("delBookbuildingPatient")
    @TokenRequired
    public BaseResponse delBookbuildingPatient(Integer id) {
        BaseResponse baseResponse =new BaseResponse();
        try {
            //查询患者的就诊记录
            LambdaQueryWrapper<LymsPcase> wrapper = new QueryWrapper().lambda();
            wrapper.eq(LymsPcase::getPid, id);
            List<LymsPcase> lymsPcaseList = lymsPcaseService.list(wrapper);
            if(CollectionUtils.isEmpty(lymsPcaseList)) {
                final boolean b = lymsPatientService.removeById(id);
                baseResponse.setErrorcode(b ? 0 : 1);
                baseResponse.setErrormsg(b ? "成功" : "失败");
            }else {
                baseResponse.setErrorcode(1);
                baseResponse.setErrormsg("该档案有关联病例，不许删除");
            }
        } catch (Exception e) {
            baseResponse.setErrorcode(1);
            baseResponse.setErrormsg("系统异常");
            e.printStackTrace();
        }
        return baseResponse;
    }
    /**
     * PC新模块建档查询列表分页
     *
     * @param patient
     * @return
     */
    @GetMapping("queryBookbuildingPatient")
    @TokenRequired
    public BaseResponse queryBookbuildingPatient(LymsPatient patient, int current, int size) {
        BaseResponse baseResponse = new BaseResponse();
        try {
            QueryWrapper<LymsPatient> query=new QueryWrapper();
            if(StringUtil.isNotEmpty(patient.getSynthesisQuery())){
                query.and(wrapper  -> wrapper .like("pname", patient.getSynthesisQuery())
                        .or().eq("enrolment_phone", patient.getSynthesisQuery())
                        .or().eq("idno", patient.getSynthesisQuery()));
            }
            if(StringUtil.isNotEmpty(patient.getStartCreatedtime())){
                query.ge("bookbuil_date", patient.getStartCreatedtime());
            }
            if(StringUtil.isNotEmpty(patient.getEndCreatedtime())){
                query.le("bookbuil_date", patient.getEndCreatedtime());
            }
            Page<LymsPatient> page=new Page<>(current,size);
            Page<LymsPatient> lymsPatientPage=lymsPatientService.page(page, query.orderByDesc("createdtime"));
            baseResponse.setObject(lymsPatientPage);
            baseResponse.setErrorcode(0);
            baseResponse.setErrormsg("成功");
        } catch (Exception e) {
            baseResponse.setErrorcode(1);
            baseResponse.setErrormsg("系统异常");
            e.printStackTrace();
        }
        return baseResponse;
    }

    /**
     * pc端获取用户信息，主要用于咨询页面获取用户头像
     * @param idno
     * @return
     */
    @GetMapping("getPatientByIdno/{idno}")
    @TokenRequired
    public BaseResponse getHeadImage(@PathVariable("idno") String idno) {
        BaseResponse baseResponse = new BaseResponse();
        try {
            QueryWrapper<LymsPatient> query=new QueryWrapper();
            query.eq("idno",idno);
            LymsPatient lymsPatient = lymsPatientService.getOne(query);
            baseResponse.setObject(lymsPatient);
            baseResponse.setErrorcode(0);
            baseResponse.setErrormsg("成功");
        } catch (Exception e) {
            log.error("根据身份证号查询用户信息异常,idno={}",idno,e);
            baseResponse.setErrorcode(1);
            baseResponse.setErrormsg("未查询到用户信息");
        }
        return baseResponse;
    }

}
