package com.lyms.talkonlineweb.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.lyms.talkonlineweb.annotation.TokenRequired;
import com.lyms.talkonlineweb.domain.LymsPermission;
import com.lyms.talkonlineweb.domain.LymsUser;
import com.lyms.talkonlineweb.result.BaseResponse;
import com.lyms.talkonlineweb.service.LymsPermissionService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 权限管理
 */

@RestController
@Log4j2
public class PermissController {
    @Autowired
    private LymsPermissionService lymsPermissionService;

    /**
     * 添加或修改权限
     * @param perm
     * @return
     */
    @PostMapping("savePerm")
    @TokenRequired
    public BaseResponse savePerm(LymsPermission perm){
        BaseResponse baseResponse=new BaseResponse();
        boolean f=lymsPermissionService.saveOrUpdate(perm);
        baseResponse.setErrorcode(f==true?0:1);
        return baseResponse;
    }

    /**
     * 删除权限
     * @param perm
     * @return
     */
    @PostMapping("delPerm")
    @TokenRequired
    public BaseResponse delPerm(LymsPermission perm){
        BaseResponse baseResponse=new BaseResponse();
        boolean f=lymsPermissionService.remove(Wrappers.query(perm));
        baseResponse.setErrorcode(f==true?0:1);
        return baseResponse;
    }

    /**
     * 获取权限列表
     * @param perm
     * @param current
     * @param size
     * @return
     */
    @PostMapping("sltPerm")
    @TokenRequired
    public BaseResponse sltPerm(LymsPermission perm,int current,int size){
        BaseResponse baseResponse=new BaseResponse();
        Page<LymsPermission> page=new Page<>(current,size);

        Page<LymsPermission> perIPage=lymsPermissionService.page(page,Wrappers.query(perm).orderByDesc("updated_time","createdtime"));
        baseResponse.setObject(perIPage);

        return baseResponse;
    }

    /**
     * 获取所有权限列表
     * @param perm
     * @return
     */
    @GetMapping("getAllPerm")
    @TokenRequired
    public BaseResponse getAllPerm(LymsPermission perm){
        BaseResponse baseResponse=new BaseResponse();

        List<LymsPermission> perIPage=lymsPermissionService.list(Wrappers.query(perm).orderByAsc("id"));
        baseResponse.setObject(perIPage);

        return baseResponse;
    }

}
