package com.lyms.talkonlineweb.task;

import com.alibaba.fastjson.JSONObject;
import com.lyms.talkonlineweb.util.Constant;
import com.lyms.talkonlineweb.util.StringUtil;
import com.lyms.talkonlineweb.util.WeiXinUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;

/**
 * 获取到access_token
 * 每5分钟获取一次token,保证长期有效。
 * （微信公众号规定2小时token失效，每天只能获取2000次。）
 */
@Component
@Log4j2
public class AccessTokenServlet extends HttpServlet implements Runnable {
    @Value("${getAccessToken.on_off}")
    public boolean on_off;//配置yml 微信公众号获取access_token(测试环境部署不要开启。会与线上环境冲突)

    public static volatile String accessToken = null;

    /**
     * 启动后开启线程每5分钟获取一次token
     * @throws ServletException
     */
    @PostConstruct//部署时开启注释，启动线程
    public void init() throws ServletException {
        if(on_off){
            new Thread(new AccessTokenServlet()).start(); //启动线程
        }else {
            log.info("测试环境不予获取access_token");
        }
    }
    public void run(){
        while (true){
            try{
                accessToken = this.getToken();
                if(null!=accessToken){
                    Thread.sleep(1000 * 7199); //获取到access_token 休眠 7199 秒,即 119.98 分钟
                }else{
                    Thread.sleep(1000 * 1); //获取的access_token为空 休眠1秒
                }
            }catch(Exception e){
                log.info("发生异常："+e.getMessage());
                e.printStackTrace();
                try{
                    Thread.sleep(1000 * 10); //发生异常休眠1秒
                }catch (Exception e1){

                }
            }
        }
    }

    /**
     * 获取token
     * 微信公众号
     * @return token
     */
    public String getToken() {
        // 授予形式
        String grant_type = "client_credential";
        // 接口地址拼接参数
        String getTokenApi = "https://api.weixin.qq.com/cgi-bin/token?grant_type=" + grant_type + "&appid=" + Constant.GZ_APP_ID
                + "&secret=" + Constant.GZ_SECRET;
        String tokenJsonStr = WeiXinUtil.repeatDoGetPost(getTokenApi, "GET", null);
        if(StringUtil.isEmpty(tokenJsonStr)){
            log.info("获取TOKEN :失败,repeatDoGetPost返回值为空>>>>>>>");
            return null;
        }
        JSONObject tokenJson = JSONObject.parseObject(tokenJsonStr);
        String token = tokenJson.get("access_token").toString();
        log.info("获取到的TOKEN : " + token);
        return token;
    }
}
