package com.lyms.talkonlineweb.task;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.lyms.talkonlineweb.domain.*;
import com.lyms.talkonlineweb.service.*;
import com.lyms.talkonlineweb.util.DateUtil;
import com.lyms.talkonlineweb.util.StringUtil;
import com.lyms.talkonlineweb.util.WeiXinUtil;
import com.mysql.cj.Query;
import lombok.Data;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.unit.DataUnit;

import java.util.*;

import static com.lyms.talkonlineweb.util.DateUtil.YYYY_MM_DD;

/**
 * 推送文章任务
 */

@Component
@Log4j2
public class PushArticleTaskData {
    @Value("${getAccessToken.on_off}")
    public boolean on_off;//配置yml 微信公众号获取access_token(测试环境部署不要开启。会与线上环境冲突)
    @Autowired
    private LymsArticleService lymsArticleService;
    @Autowired
    private PatientInfoService patientInfoService;
    @Autowired
    private LymsPushedartService lymsPushedartService;//推送的历史记录
    @Autowired
    private PushArticleService pushArticleService;//推送文章新逻辑shiy改
    @Autowired
    LymsPushMessagesService lymsPushMessagesService;
    @Autowired
    private LymsDictService lymsDictService;
    @Autowired
    public PushArticleTask pushArticleTask;
    @Autowired
    public LymsPushAttentionRecordService lymsPushAttentionRecordService;
    @Autowired
    private LymsPatientAttentionService lymsPatientAttentionService;
    @Autowired
    private LymsPatientService lymsPatientService;//患者



    /**
     * 每天下17点执行-要推送的文章存到数据库
     */
    @Scheduled(cron = "0 0 17 * * ?")
//    @Scheduled(cron = "0 */3 * * * ?")//测试用
    public void pushArtcleData(){
        if(!on_off){
            return;
        }
        Map<String,Object> param=new HashMap<>();
        param.put("vtype",999);
        List<LymsDict> dcLst=lymsDictService.listByMap(param);
        if (dcLst.size()>0 && dcLst.get(0).getCode()==1){
            //防止重复
            QueryWrapper<LymsPushMessages> queryWrapper2 = new QueryWrapper<>();
            queryWrapper2.apply("TO_DAYS(created_time) = TO_DAYS(NOW()) and state in (0,1,2)");
            List<LymsPushMessages> list = lymsPushMessagesService.list(queryWrapper2);
            if(!CollectionUtils.isEmpty(list)){
                log.info("要推送的文章已存在不需要再次推送>>>>>>");
                return;
            }
            log.info("开始筛选"+(DateUtil.getDateTime(new Date(), YYYY_MM_DD))+"要推送的文章存入mysql>>>>>>");
            //筛选出需要推送的文章信息-（逻辑详见视图）
            List<PushArticle> pushArticleList=pushArticleService.list();
            for (PushArticle article : pushArticleList) {
                try {
                    //需要推送文章的对象
                    LymsPushMessages lymsPushMessages=new LymsPushMessages();
                    BeanUtils.copyProperties(article,lymsPushMessages);
                    //查询推已推送的文章（如果null表示需要推送文章和短消息1，否则需要推送短消息2）
                    QueryWrapper<LymsPushedart> queryWrapper=new QueryWrapper<>();
                    queryWrapper.eq("pid", lymsPushMessages.getPid());
                    queryWrapper.eq("aid", lymsPushMessages.getAid());
                    List<LymsPushedart> lymsPushedartList = lymsPushedartService.list(queryWrapper);
                    if(!CollectionUtils.isEmpty(lymsPushedartList)&&lymsPushedartList.size()>1){
                        log.info("数据不同步，不需要推送>>>>>>");
                        return;
                    }
                    if(CollectionUtils.isNotEmpty(lymsPushedartList)){
                        //标记要推送的记录需要推送短消息2
                        lymsPushMessages.setIsweixinTwo(1);
                        //存入已推送文章列表id
                        lymsPushMessages.setPushedartId(lymsPushedartList.get(0).getId());
                        //子女关注的患者要推送的短消息
                        pushPatientAttention(lymsPushMessages);
                    }else {
                        //标记要推送的记录需要推送短消息1
                        lymsPushMessages.setIsweixinOne(1);
                        //需要推送的短消息1
                        lymsPushMessagesService.save(lymsPushMessages);
                        //子女关注的患者要推送的短消息
                        pushPatientAttention(lymsPushMessages);
                    }
                    //标记要推送的文章-短消息1标记为0说明这是推送的文章
                    lymsPushMessages.setIsweixinOne(0);
                    //保存到要推送的表lyms_push_messages需要推送的文章
                    lymsPushMessagesService.save(lymsPushMessages);
                } catch (BeansException e) {
                    e.printStackTrace();
                }
            }
        }
//        pushArticleTask.pushArtcle();//测试用

    }
    //子女关注的患者要推送的短消息,数据构建
    public void pushPatientAttention(LymsPushMessages lymsPushMessages){
        if (null!=lymsPushMessages.getPid()) {
            final List<LymsPatientAttention> lymsPatientAttentions = lymsPatientAttentionService.list(new QueryWrapper<LymsPatientAttention>().eq("attention_id", lymsPushMessages.getPid()));
            for (LymsPatientAttention lymsPatientAttention : lymsPatientAttentions) {
                final LymsPatient patient = lymsPatientService.getOne(new QueryWrapper<LymsPatient>().eq("id", lymsPatientAttention.getPid()));
                if (null!=patient.getId()) {
                    LymsPushAttentionRecord attentionRecord=new LymsPushAttentionRecord();
                    attentionRecord.setPid(patient.getId());
                    attentionRecord.setPGzopenid(patient.getGzopenid());
                    attentionRecord.setPaId(lymsPushMessages.getPid());
                    if(null!=lymsPushMessages.getIsweixinOne() && 1==lymsPushMessages.getIsweixinOne()) {
                        attentionRecord.setPaIsweixinType(1);
                        attentionRecord.setPaWeixText(lymsPushMessages.getWeixTextOne());
                    }else  {
                        attentionRecord.setPaIsweixinType(2);
                        attentionRecord.setPaWeixText(lymsPushMessages.getWeixTextTwo());
                    }
                    lymsPushAttentionRecordService.save(attentionRecord);
                }

            }
        }
    }
}


