package com.lyms.talkonlineweb.task;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.lyms.talkonlineweb.domain.*;
import com.lyms.talkonlineweb.enums.VisitEnum;
import com.lyms.talkonlineweb.service.*;
import com.lyms.talkonlineweb.util.*;
import lombok.Data;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * 回访信息回复
 */

@Component
@Log4j2
public class PushChatInfoTask {
    @Value("${hx.hxuser}")
    private String hxuser;
    @Autowired
    private LymsArticleService lymsArticleService;
    @Autowired
    private PatientInfoService patientInfoService;
    @Autowired
    private LymsPushedartService lymsPushedartService;//推送的历史记录
    @Autowired
    private PushArticleService pushArticleService;//推送文章新逻辑shiy改
    @Autowired
    private LymsPatientService lymsPatientService;
    @Autowired
    private LymsPushMessagesService lymsPushMessagesService;
    @Autowired
    private LymsPcaseService lymsPcaseService;//病例
    @Autowired
    private LymsDoctorService lymsDoctorService;
    @Autowired
    public LymsPushAttentionRecordService lymsPushAttentionRecordService;
    @Autowired
    private LymsChatgroupService lymsChatgroupService;
    @Autowired
    private LymsReturnVisitRecordService lymsReturnVisitRecordService;
    @Autowired
    private HXService hxService;
    @Autowired
    private LymsIllnessService lymsIllnessService;
    @Autowired
    private LymsChatInfoService lymsChatInfoService;
    @Autowired
    private LymsMessageService lymsMessageService;

    @Autowired
    private LymsTkrecordService lymsTkrecordService;
    @Autowired
    private LymsPushVisitRecordService lymsPushVisitRecordService;
    @Autowired
    private PushArticleTask pushArticleTask;
    @Value("${getAccessToken.on_off}")
    public boolean on_off;//配置yml 微信公众号获取access_token(测试环境部署不要开启。会与线上环境冲突)

    /**
     * 每天19点执行自动回访功能
     *  第二天：就诊医生
     *  第五天：值班护士
     *  第十天：值班医生
     *  第二十天：就诊医生
     *  第三十天：值班医生
     */
    @Scheduled(cron = "0 0 19 * * ?")
    public void pushChatInfo() {
        if(!on_off){
            return;
        }
        log.info("自动回访功能定时任务,开始....");
        //用户下的病例信息。每个环信群组代表一个病例，一个病例下有多个疾病种类。
        List<Map<String,Object>> PcInfoList=lymsPatientService.getPcInfoList();
        log.info("自动回访功能定时任务,病例数size = {}",PcInfoList.size());
        for (Map<String, Object> map : PcInfoList) {

            try {
                //患者信息
                LymsPatient patient= lymsPatientService.getOne(new QueryWrapper<LymsPatient>()
                        .lambda().eq(LymsPatient::getIdno, map.get("idno").toString()));
                //医生信息
                LymsDoctor doctor= lymsDoctorService.getOne(new QueryWrapper<LymsDoctor>()
                        .lambda().eq(LymsDoctor::getDlogin, map.get("dlogin").toString()));

                //计算（自动回复周期上传病例后- 1：第二天2：第五天3：第十天4：第二十天5：第三十天）
                int day = DateUtil.daysBetween(DateUtil.parseYMD(map.get("createdtime").toString()),new Date());
                log.info("自动回访功能定时任务,pcid={},day={}",map.get("pcid"),day);
                switch (day) {
                    case 1:
                        getEverydayInfo(map,patient,doctor,0,1);
                        break;
                    case 4:
                        getEverydayInfo(map,patient,doctor,2,2);
                        break;
                    case 9:
                        getEverydayInfo(map,patient,doctor,1,3);
                        break;
                    case 19:
                        getEverydayInfo(map,patient,doctor,0,4);
                        break;
                    case 29:
                        getEverydayInfo(map,patient,doctor,1,5);
                        break;
                }
            } catch (Exception e) {
                log.error(e);
            }

        }
        log.info("自动回访功能定时任务,end.....");
    }

    /**
     * 每天推送逻辑
     * @param map
     * @param patient
     * @param doctor
     * @param adminType 0:就诊医生 1：值班医生 2:值班护士
     * @param ChatInfoType 自动回复周期上传病例后- 1：第二天2：第五天3：第十天4：第二十天5：第三十天
     */
    @Transactional(rollbackFor = {Exception.class,RuntimeException.class})
    public void getEverydayInfo(Map<String,Object> map,LymsPatient patient,LymsDoctor doctor,int adminType,int ChatInfoType) throws Exception,RuntimeException {
        try {
            //值班医生/值班护士信息
            List<LymsDoctor> doctorAminList= lymsDoctorService.list(new QueryWrapper<LymsDoctor>()
                    .lambda().eq(LymsDoctor::getDpid, doctor.getDpid())
                    .in(LymsDoctor::getAdminType, Arrays.asList(1,2))
                    .orderByAsc(LymsDoctor::getAdminType));
            //获取 就诊医生/值班医生/值班护士账号
            LymsDoctor doctor2=null;
            if(adminType!=0) {
                for (LymsDoctor lymsDoctor : doctorAminList) {
                    if (lymsDoctor.getAdminType() == adminType) {
                        doctor2 = lymsDoctor;
                        break;
                    }
                }
                if (null==doctor2){
                    doctor2=doctor;
                }
            }else {
                doctor2=doctor;
            }
            log.info("自动回访功能定时任务,pcid={},doctor={}",map.get("pcid"),doctor2);
            //环信群组
            LymsChatgroup chatgroup=null;
            //病例下的疾病
            final List<LymsIllness> lymsIllnessList = lymsIllnessService.list(new QueryWrapper<LymsIllness>()
                    .lambda().eq(LymsIllness::getPcid, (Integer) map.get("pcid")));
            for (LymsIllness lymsIllness : lymsIllnessList) {
                //要推送的疾病对应的话术
                final LymsChatInfo chatInfo = lymsChatInfoService.getOne(new QueryWrapper<LymsChatInfo>()
                        .lambda().eq(LymsChatInfo::getIllid, lymsIllness.getIid())
                        .eq(LymsChatInfo::getType, ChatInfoType));
                ///推送回访
                if (null!=chatInfo && null!=doctor2) {
                    //环信群组信息
                    chatgroup = lymsChatgroupService.getOne(new QueryWrapper<LymsChatgroup>()
                            .lambda().eq(LymsChatgroup::getPcid, (Integer)map.get("pcid")));
                    if(null==chatgroup){
                        log.info("自动回访功能定时任务,pcid={},开始创建聊天组...",map.get("pcid"));
                        //创建聊天群组
                        List<String> adminDlogins=new ArrayList<>();
                        adminDlogins.add(patient.getIdno());
                        List<String> groupnames=new ArrayList<>();
                        groupnames.add(patient.getPname());
                        //患者注册环信
                        if (StringUtil.isEmpty(patient.getHxid())) {
                            lymsPatientService.addPatientHxId(patient);
                        }
                        //科室值班医生注册环信
                        for (LymsDoctor lymsDoctor : doctorAminList) {
                            if(StringUtil.isEmpty(lymsDoctor.getHxid())) {
                                lymsDoctorService.addDoctorHxId(lymsDoctor);
                            }
                            adminDlogins.add(lymsDoctor.getDlogin());
                            groupnames.add(lymsDoctor.getDname());
                        }
                        if (!adminDlogins.contains(doctor.getDlogin())) {
                            //医生注册环信
                            if (StringUtil.isEmpty(doctor.getHxid())) {
                                lymsDoctorService.addDoctorHxId(doctor);
                            }
                            adminDlogins.add(doctor.getDlogin());
                            groupnames.add(doctor.getDname());
                        }
                        JSONObject rJson = hxService.addChatGroups(StringUtils.join(adminDlogins.toArray(), ","), hxuser, adminDlogins.toArray(new String[adminDlogins.size()]));
                        LymsChatgroup group =new LymsChatgroup();
                        group.setGroupname(StringUtils.join(groupnames.toArray(), ","));
                        group.setDescription(StringUtils.join(adminDlogins.toArray(), ","));
                        group.setOwnerk(hxuser);
                        group.setFromp(patient.getIdno());
                        group.setHxgroupid(rJson.getJSONObject("data").getString("groupid"));
                        group.setTarget(doctor2.getDlogin());
                        group.setType(1);
                        group.setStat(0);
                        group.setPcid((Integer)map.get("pcid"));
                        group.setHdepartId(doctor2.getDpid());
                        final boolean save = lymsChatgroupService.save(group);
                        if (!save) {
                            throw new Exception("回滚状态，执行失败，请联系管理员");
                        }
                        chatgroup=group;
                    }
                    log.info("自动回访功能定时任务,pcid={},chatgroup={}",map.get("pcid"),chatgroup);
                    //推送回访消息和保存发送记录
                    JSONObject jsonObject = hxService.sendGroupMsg(new String[]{chatgroup.getHxgroupid()}, chatInfo.getConten(), doctor2.getDlogin());
                    log.info("自动回访功能定时任务,pcid={},推送回访消息返回值={}",map.get("pcid"),jsonObject);
                    if (null!=jsonObject) {
                        //保存发送消息记录
                        LymsMessage message=new LymsMessage();
                        message.setContent(chatInfo.getConten());
                        message.setFromid(doctor2.getDlogin());
                        message.setTargetid(chatgroup.getHxgroupid());
                        message.setMtype("TEXT");
                        message.setSendtime(new Date());
                        message.setType(1);
                        final boolean save = lymsMessageService.save(message);
                        if (! save) {
                            throw new Exception("自动回访功能定时任务，执行失败，请联系管理员");
                        }
                    }else {
                        throw new Exception("自动回访功能定时任务，执行失败，请联系管理员");
                    }
                    //同时推送公众号消息
                    Map<String,Object> mapTemplate=new HashMap<>();
                    String content=chatInfo.getConten().substring(0,50)+"...";
                    mapTemplate.put("first",new DataEntity(content,"#173177"));
                    mapTemplate.put("keyword1",new DataEntity(doctor2.getDname(),"#173177"));
                    mapTemplate.put("keyword2",new DataEntity(DateUtil.getyyyy_MM_dd1(new Date()),"#173177"));
                    mapTemplate.put("remark",new DataEntity("请点击下方进入小程序，根据病例点咨询查看医生回访信息。","#173177"));
                    Map<String,Object> mapInfo = pushArticleTask.pLoginInfo(patient.getId());
                    Integer code= WeiXinUtil.SendWeChatMsg(patient.getGzopenid(),Constant.GZ_TEMPLATE_ID_DOCTOR_VISIT,mapTemplate,mapInfo);
                    //保存推送记录
                    LymsPushVisitRecord lymsPushVisitRecord=new LymsPushVisitRecord();
                    lymsPushVisitRecord.setPid(patient.getId());
                    lymsPushVisitRecord.setPname(patient.getPname());
                    lymsPushVisitRecord.setIid(lymsIllness.getIid());
                    lymsPushVisitRecord.setIiname(lymsIllness.getIname());
                    lymsPushVisitRecord.setDid(doctor2.getDid());
                    lymsPushVisitRecord.setDname(doctor2.getDname());
                    lymsPushVisitRecord.setType(chatInfo.getType());
                    lymsPushVisitRecord.setContent(chatInfo.getConten());
                    lymsPushVisitRecord.setState((null==code||code!=0)?2:1);//推送状态： 1成功 2失败。
                    lymsPushVisitRecord.setRemark("推送短消息-> " + VisitEnum.getName(chatInfo.getType())+" - "+patient.getPname()+" ;回访医生:"+doctor2.getDname()+";疾病名称："+ lymsIllness.getIname() + "; [ "+(lymsPushVisitRecord.getState()==1?"成功":"失败")+"]; code:" + code);
                    lymsPushVisitRecordService.save(lymsPushVisitRecord);
                }else {
                    throw new Exception("自动回访功能定时任务,执行失败，没有疾病话术。");
                }
            }
            //增加回访记录，修改环信群组状态，修改问诊记录状态
            LymsReturnVisitRecord returnVisitRecord=new LymsReturnVisitRecord();
            returnVisitRecord.setDlogin(doctor2.getDlogin());
            returnVisitRecord.setDpid(doctor2.getDpid());
            returnVisitRecord.setIdno(patient.getIdno());
            //0 自动回访 1 医生回访
            returnVisitRecord.setType(0);
            returnVisitRecord.setHxgroupid(chatgroup.getHxgroupid());
            returnVisitRecord.setPcid(chatgroup.getPcid());
            returnVisitRecord.setCreatedtime(new Date());
            final boolean save = lymsReturnVisitRecordService.save(returnVisitRecord);
            if (!save) {
                throw new Exception("回滚状态，执行失败，请联系管理员");
            }
            chatgroup.setType(1);
            chatgroup.setStat(0);
            final boolean b = lymsChatgroupService.updateById(chatgroup);
            if (!b) {
                throw new Exception("回滚状态，执行失败，请联系管理员");
            }
            //修改同一群组记录的状态
            /*final List<LymsTkrecord> tkrecordList = lymsTkrecordService.list(new QueryWrapper<LymsTkrecord>()
                    .lambda().eq(LymsTkrecord::getHxgroupid, chatgroup.getHxgroupid()));
            for (LymsTkrecord lymsTkrecord : tkrecordList) {
                lymsTkrecord.setStat(0);
                final boolean saveOrUpdate = lymsTkrecordService.updateById(lymsTkrecord);
                if (!saveOrUpdate) {
                    throw new Exception("回滚状态，执行失败，请联系管理员");
                }
            }*/
        } catch (Exception e) {
            log.error(e);
        }
    }

}

