package com.lyms.talkonlineweb.task;

import com.baomidou.mybatisplus.core.conditions.query.Query;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.lyms.talkonlineweb.domain.LymsPushIllnessMsg;
import com.lyms.talkonlineweb.domain.LymsPushMessages;
import com.lyms.talkonlineweb.result.PushIllnessTypeDataResult;
import com.lyms.talkonlineweb.service.*;
import com.lyms.talkonlineweb.util.DateUtil;
import com.lyms.talkonlineweb.util.StringUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.scheduling.annotation.Scheduled;

import java.util.*;

/**
 * 疾病分类推送数据整理（1普通疾病，2慢性病）
 * 慢性病：确诊后第一个月推送一次，间隔3个月进行第二次推送
 * 普通疾病：确诊15天后 推送
 */

@Component
@Log4j2
public class PushIllnessTypeData {

    @Autowired
    private LymsPushIllnessMsgService lymsPushIllnessMsgService;
    @Value("${getAccessToken.on_off}")
    public boolean on_off;//配置yml 微信公众号获取access_token(测试环境部署不要开启。会与线上环境冲突)


    /**
     * 每天00:00点执行-要推送的公众号信息数据整理
     */
    @Scheduled(cron = "0 0 0 * * ?")
    public void PushIllnessTypeData(){
        if(!on_off){
            return;
        }
        Map param=new HashMap<>();
        param.put("push_type", 0);
        final List<LymsPushIllnessMsg> list = lymsPushIllnessMsgService.listByMap(param);
        if(CollectionUtils.isNotEmpty(list)){
            log.info("要推送的文章已存在不需要再次推送>>>>>>");
            return;
        }
        List<PushIllnessTypeDataResult> result= lymsPushIllnessMsgService.getPatientIllenssType();
        for (PushIllnessTypeDataResult dataResult : result) {
            LymsPushIllnessMsg lymsPushIllnessMsg = new LymsPushIllnessMsg();
            try {
                if(null!=dataResult.getCreatedtime() && StringUtil.isNotEmpty(dataResult.getGzopenid()) &&
                    StringUtil.isNotEmpty(dataResult.getPhone()) && StringUtil.isNotEmpty(dataResult.getPasswd())) {

                    //这里要提前一天提醒。
                    long newDate = DateUtil.getYmdDate(new Date()).getTime();
                    long illenssDate_15 = DateUtil.addDay(DateUtil.getYmdDate(dataResult.getCreatedtime()), 14).getTime();
                    long illenssDate_30 = DateUtil.addDay(DateUtil.getYmdDate(dataResult.getCreatedtime()), 29).getTime();
                    long illenssDate_120 = DateUtil.addDay(DateUtil.getYmdDate(dataResult.getCreatedtime()), 119).getTime();

                    //普通疾病
                    if (1== dataResult.getIllnessType() ) {
                        if(null==dataResult.getPushType()) {//普通疾病要么是null 第一次推。要么是2失败。不会出现1的时候
                            if (newDate != illenssDate_15) {
                                continue;
                            }
                        }
                        lymsPushIllnessMsg.setIllnessType(1);
                        String text = "患者接受治疗疾病症状应逐渐消退，症状减轻后药量需要减少，及时来医院复诊可以让医生调整用药量，" +
                                "如果患者出现症状加重的情况需及时来医院调整药物用量或更换药物种类。" +
                                "用药请按照医嘱进行，不要自行增加或减少药物用量，症状消失后可来院复诊检查是否需要用药维持";
                        lymsPushIllnessMsg.setContent(text);
                    }
                    //慢性疾病
                    if (2== dataResult.getIllnessType()) {
                        if(null==dataResult.getPushType()){
                            if(newDate != illenssDate_30){
                                continue;
                            }
                            dataResult.setPushCount(1);
                        }else {
                            if(1==dataResult.getPushType()){
                                if(newDate != illenssDate_120){
                                    continue;
                                }
                                dataResult.setPushCount(2);
                            }
                        }
                        lymsPushIllnessMsg.setIllnessType(2);
                        lymsPushIllnessMsg.setPushCount(dataResult.getPushCount());
                        String text="患者出院后疾病症状应逐渐减轻，如果患者出现症状加重或症状反复，需立即前往医院就诊。" +
                                "用药请按医嘱服用药物，不要自行增添或减少药物用量，中间不要停药，药物不足时可自行来医院复诊取药。";
                        lymsPushIllnessMsg.setContent(text);
                    }

                    lymsPushIllnessMsg.setPid(dataResult.getId());
                    lymsPushIllnessMsg.setPhone(dataResult.getPhone());
                    lymsPushIllnessMsg.setPasswd(dataResult.getPasswd());
                    lymsPushIllnessMsg.setGzopenid(dataResult.getGzopenid());
                    lymsPushIllnessMsg.setPname(dataResult.getPname());
                    lymsPushIllnessMsg.setPcid(dataResult.getPcid());
                    lymsPushIllnessMsg.setHname(dataResult.getHname());
                    lymsPushIllnessMsg.setDname(dataResult.getDname());
                    lymsPushIllnessMsg.setDtname(dataResult.getDtname());
                    lymsPushIllnessMsg.setCode(dataResult.getCode());
                    lymsPushIllnessMsg.setValue(dataResult.getValue());
                    if(null!=dataResult.getMid()){//存在id是要更新推送记录，失败或慢性病第二次推送
                        lymsPushIllnessMsg.setId(dataResult.getMid());
                        lymsPushIllnessMsg.setPushType(0);
                    }
                    lymsPushIllnessMsgService.saveOrUpdate(lymsPushIllnessMsg);
                }else {
                    log.info("推送者数据有空值，不予推送。>>>>>>");
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

    }

}


