package com.lyms.talkonlineweb.task;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.lyms.talkonlineweb.domain.*;
import com.lyms.talkonlineweb.service.*;
import com.lyms.talkonlineweb.util.Constant;
import com.lyms.talkonlineweb.util.DateUtil;
import com.lyms.talkonlineweb.util.StringUtil;
import com.lyms.talkonlineweb.util.WeiXinUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.scheduling.annotation.Scheduled;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.lyms.talkonlineweb.util.DateUtil.YYYY_MM_DD;

/**
 * 疾病分类推送（1普通疾病，2慢性病）
 * 慢性病：确诊后第一个月推送一次，间隔3个月进行第二次推送
 * 普通疾病：确诊15天后 推送
 */

@Component
@Log4j2
public class PushIllnessTypeTask {

    @Autowired
    private LymsPushIllnessMsgService lymsPushIllnessMsgService;
    @Value("${getAccessToken.on_off}")
    public boolean on_off;//配置yml 微信公众号获取access_token(测试环境部署不要开启。会与线上环境冲突)


    /**
     * 每天下18:30点执行-要推送的公众号信息
     */
    @Scheduled(cron = "0 30 18 * * ?")
    public void PushIllnessTypeTask(){
        if(!on_off){
            return;
        }
        Map param=new HashMap<>();
        param.put("push_type", 0);
        final List<LymsPushIllnessMsg> list = lymsPushIllnessMsgService.listByMap(param);
        if(CollectionUtils.isEmpty(list)){
            log.info("没有要推送的文章。>>>>>>");
            return;
        }
        for (LymsPushIllnessMsg lymsPushIllnessMsg : list) {
            try {
                //推送微信公众号消息
                Map<String,Object> map=new HashMap<>();
                map.put("first",new DataEntity(lymsPushIllnessMsg.getContent(),"#173177"));
                map.put("keyword1",new DataEntity(lymsPushIllnessMsg.getPname(),"#173177"));
                map.put("keyword2",new DataEntity(lymsPushIllnessMsg.getHname(),"#173177"));
                map.put("keyword3",new DataEntity(lymsPushIllnessMsg.getDname(),"#173177"));
                map.put("keyword4",new DataEntity(lymsPushIllnessMsg.getDtname(),"#173177"));
                map.put("keyword5",new DataEntity(lymsPushIllnessMsg.getValue(),"#173177"));
                String illnessType="";
                if(null!=lymsPushIllnessMsg.getIllnessType()){
                    illnessType=(1==lymsPushIllnessMsg.getIllnessType()?"普通疾病":"慢性病");
                }
                map.put("remark",new DataEntity(illnessType+"，请遵医嘱及时复诊。","#173177"));
                Map<String,Object> mapInfo=new HashMap<>();
                map.put("plogin", StringUtil.isEmpty(lymsPushIllnessMsg.getPhone())?"":lymsPushIllnessMsg.getPhone());
                map.put("passwd", StringUtil.isEmpty(lymsPushIllnessMsg.getPasswd())?"":lymsPushIllnessMsg.getPasswd());

                Integer code= WeiXinUtil.SendWeChatMsg(lymsPushIllnessMsg.getGzopenid(), Constant.GZ_TEMPLATE_ID_RETURN_VISIT,map,mapInfo);
                if(null==code||code!=0){
                    //更新到 LymsPushIllnessMsg 记录
                    lymsPushIllnessMsg.setPushType(2);//推送状态：0待推送 1成功 2失败。（在备注写失败原因）
                    lymsPushIllnessMsg.setPushResult("推送消息"+illnessType+"失败。。。"+lymsPushIllnessMsg.getPname()+"; code:"+code);
                    lymsPushIllnessMsg.setPushTime(new Date());
                    log.info("推送失败！"+lymsPushIllnessMsg.getPname()+"; code:"+code);
                }else {
                    //更新到 LymsPushIllnessMsg 记录
                    lymsPushIllnessMsg.setPushType(1);//推送状态：0待推送 1成功 2失败。（在备注写失败原因）
                    lymsPushIllnessMsg.setPushResult(lymsPushIllnessMsg.getPname()+"推送消息"+illnessType+"成功。");
                    lymsPushIllnessMsg.setPushTime(new Date());
                }
                lymsPushIllnessMsgService.updateById(lymsPushIllnessMsg);
            } catch (Exception e) {
                //出现未知错误也要把需要推送的信息置为失败。保证第二天还可以继续推送。因为构造数据时用0判断的。
                lymsPushIllnessMsg.setPushType(2);//推送状态：0待推送 1成功 2失败。（在备注写失败原因）
                lymsPushIllnessMsg.setPushResult("推送消息失败。。。"+lymsPushIllnessMsg.getPname()+"; error:"+e.getMessage());
                lymsPushIllnessMsg.setPushTime(new Date());
                lymsPushIllnessMsgService.updateById(lymsPushIllnessMsg);
                log.info(e.getMessage());
                e.printStackTrace();
            }

        }
    }

}


