package com.lyms.talkonlineweb.util;

import lombok.extern.slf4j.Slf4j;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * @ProjectName: talkonline
 * @Package: com.lyms.talkonlineweb.util
 * @ClassName: DateUtil
 * @Author: lqy
 * @Description: 时间格式类
 * @Date: 2021-09-07 17:53
 * @Version:
 */
public class DateUtil {
    /**
     * 缓存时间格式化
     */
    private static ConcurrentHashMap<String, DateFormat> dateFormatCache = new ConcurrentHashMap<String, DateFormat>();
    private static Lock lock = new ReentrantLock();

    public static final String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";
    public static final String YYYY_MM_DD = "yyyy-MM-dd";
    public static SimpleDateFormat dd = new SimpleDateFormat("dd");
    public static SimpleDateFormat yyyy = new SimpleDateFormat("yyyy");
    public static SimpleDateFormat yyyyMM = new SimpleDateFormat("yyyyMM");
    public static SimpleDateFormat ymd = new SimpleDateFormat("yyyyMMdd");
    public static SimpleDateFormat y_m_d = new SimpleDateFormat("yyyy-MM-dd");
    public static SimpleDateFormat y_m = new SimpleDateFormat("yyyy年MM月");

    public static SimpleDateFormat y_m_d1 = new SimpleDateFormat("yyyy年MM月dd日");
    public static SimpleDateFormat y_m_d2 = new SimpleDateFormat("yyyy年MM月dd日 HH点mm分ss秒");
    public static SimpleDateFormat md = new SimpleDateFormat("MM-dd");
    public static SimpleDateFormat m_d = new SimpleDateFormat("MM/dd");
    public static SimpleDateFormat y_m_d_h_m_s = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    public static SimpleDateFormat y_m_d_h_m1 = new SimpleDateFormat("yyyy-MM-dd HH:mm");
    public static SimpleDateFormat y_m_d_h = new SimpleDateFormat("yyyy-MM-dd HH");
    public static String[] parsePatterns = {"yyyy-MM-dd", "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd HH:mm", "yyyy/MM/dd", "yyyy/MM/dd HH:mm:ss",
            "yyyy/MM/dd HH:mm"};

    public static SimpleDateFormat yyyyMMddHHmmss = new SimpleDateFormat("yyyyMMddHH:mm:ss");

    public static SimpleDateFormat yyyy_mm = new SimpleDateFormat("yyyy-MM");

    public static SimpleDateFormat y_m_d_h_m = new SimpleDateFormat("yyyyyMMddHHmm");
    public static SimpleDateFormat yyyyMMddHHmmssSSS = new SimpleDateFormat("yyyyMMddHHmmssSSS");
    public static final int DAY_SECONDS = 86399;
    public static final int DAY_FULL_SECONDS = 86400;


    /**
     * 私有构造方法，禁止对该类进行实例化
     */
    private DateUtil() {
    }

    public static String getYyyyMmDdHhMmSs(Date datetime) {
        return getDateTime(datetime, YYYY_MM_DD_HH_MM_SS);
    }
    public static String getyyyy_MM_dd1(Date d) {
        if (d == null) {
            return null;
        }
        try {
            return y_m_d1.format(d);
        } catch (Exception e) {
            return null;
        }
    }
    public static String getyyyy_MM_dd_HH_mm_ss(Date d) {
        if (d == null) {
            return null;
        }
        try {
            return y_m_d2.format(d);
        } catch (Exception e) {
            return null;
        }
    }
    public static Date parseYMD(String s) {
        if (s == null) {
            return null;
        }
        lock.lock();
        try {

            return y_m_d.parse(s);
        } catch (Exception e) {
            return null;
        } finally {
            lock.unlock();
        }
    }
    public static Date parseYMDHMS(String s) {
        if (s == null) {
            return null;
        }
        lock.lock();
        try {

            return y_m_d_h_m_s.parse(s);
        } catch (Exception e) {
            return null;
        } finally {
            lock.unlock();
        }
    }
    public static Date parseYMDHM(String s) {
        if (s == null) {
            return null;
        }
        lock.lock();
        try {

            return y_m_d_h_m1.parse(s);
        } catch (Exception e) {
            return null;
        } finally {
            lock.unlock();
        }
    }

    public static String getDateTime(Date date, String pattern) {
        if (date == null) {
            return "";
        }

        DateFormat format = getDateFormat(pattern);
        lock.lock();
        try {
            return format.format(date);
        } finally {
            lock.unlock();
        }
    }

    public static DateFormat getDateFormat(String pattern) {
        if (StringUtil.isEmpty(pattern)) {
            pattern = YYYY_MM_DD;
        }

        if (dateFormatCache.get(pattern) != null) {
            return dateFormatCache.get(pattern);
        }
        lock.lock();
        try {
            DateFormat format = new SimpleDateFormat(pattern);
            dateFormatCache.put(pattern, format);
            return format;
        } catch (Exception e) {
        } finally {
            lock.unlock();
        }
        return null;
    }

    public static String getSeqString() {
        SimpleDateFormat fm = new SimpleDateFormat("yyMMddHHmmss");
        return fm.format(new Date());
    }

    /**
     * 时分秒清 yyyy-MM-dd 00:00:00
     * @param date
     * @return
     */
    public static Date getYmdDate(Date date) {
        try {
            date.setTime(new SimpleDateFormat("yyyy-MM-dd").parse(y_m_d.format(date)).getTime());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return date;
    }

    /**
     * 传入时间，增加天数
     * @param srcDate
     * @param days
     * @return
     */
    public static Date addDay(Date srcDate, int days) {
        lock.lock();
        try {

            Calendar rightNow = Calendar.getInstance();
            rightNow.setTime(srcDate);
            rightNow.add(Calendar.DAY_OF_YEAR, days);
            return rightNow.getTime();
        } finally {
            lock.unlock();
        }
    }
    public static Integer getAge(Date birth) {
        if (null == birth) {
            return null;
        }
        java.util.Calendar calendar = Calendar.getInstance();
        Date date = new Date();
        calendar.setTime(date);

        Calendar c1 = Calendar.getInstance();
        Calendar c2 = Calendar.getInstance();
        c1.setTime(date);
        c2.setTime(birth);
        int year1 = c1.get(Calendar.YEAR);
        int year2 = c2.get(Calendar.YEAR);

        int mouth1 = c1.get(Calendar.MONTH);//0当前
        int mouth2 = c2.get(Calendar.MONTH);//4
        int i = 0;

        if (mouth2 == mouth1) {
            int days1 = c1.get(Calendar.DAY_OF_MONTH);//当前10
            int days2 = c2.get(Calendar.DAY_OF_MONTH);//1
            if (days1 < days2) {
                i = i - 1;
            } else if (days1 > days2) {
            }
        } else if (mouth2 > mouth1) {
            i = i - 1;
        }

        return (Math.abs(year1 - year2) + i);
    }

    /**
     * 时间差
     * @param start
     * @param end
     * @return
     */
    public static int daysBetween(Date smdate, Date bdate) {
        if (smdate == null || bdate == null) {
            return -1;
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        try {
            smdate = sdf.parse(sdf.format(smdate));
            bdate = sdf.parse(sdf.format(bdate));
            Calendar cal = Calendar.getInstance();
            cal.setTime(smdate);
            long time1 = cal.getTimeInMillis();
            cal.setTime(bdate);
            long time2 = cal.getTimeInMillis();
            long between_days = (time2 - time1) / (1000 * 3600 * 24);
            return Integer.parseInt(String.valueOf(between_days));
        } catch (ParseException e) {
            e.printStackTrace();
        }

        return 0;
    }
    /**
     * 当前的开始时间
     *
     * @param date
     * @return
     */
    public static Date getDayFirstSecond(Date date) {
        Calendar instance = Calendar.getInstance();
        instance.setTime(date);
        instance.set(Calendar.HOUR_OF_DAY, 0);
        instance.set(Calendar.MINUTE, 0);
        instance.set(Calendar.SECOND, 0);
        instance.set(Calendar.MILLISECOND, 0);
        return instance.getTime();
    }

    /**
     * 把传进来的日期 设置为当天的最后一秒
     *
     * @param date
     */
    public static Date getDayLastSecond(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        return calendar.getTime();
    }

}
